'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var core = require('@material-ui/core');
var createMuiTheme = _interopDefault(require('@material-ui/core/styles/createMuiTheme'));
var reactSimpleHookStore = require('react-simple-hook-store');
var CloseIcon = _interopDefault(require('@material-ui/icons/Close'));
var icons = require('@material-ui/icons');
var reactHookForm = require('react-hook-form');
var yup = require('yup');
var styles = require('@material-ui/styles');
var reactDnd = require('react-dnd');
var useMediaQuery = _interopDefault(require('@material-ui/core/useMediaQuery'));
var purple = _interopDefault(require('@material-ui/core/colors/purple'));
var green = _interopDefault(require('@material-ui/core/colors/green'));
var reactDndHtml5Backend = require('react-dnd-html5-backend');
var CssBaseline = _interopDefault(require('@material-ui/core/CssBaseline'));
var ThemeProvider = _interopDefault(require('@material-ui/styles/ThemeProvider'));
var yup$1 = require('@hookform/resolvers/yup');
var styles$1 = require('@material-ui/core/styles');
var Paper = _interopDefault(require('@material-ui/core/Paper'));
var InputBase = _interopDefault(require('@material-ui/core/InputBase'));
var IconButton = _interopDefault(require('@material-ui/core/IconButton'));
var MenuIcon = _interopDefault(require('@material-ui/icons/Menu'));
var SearchIcon = _interopDefault(require('@material-ui/icons/Search'));
var Popper = _interopDefault(require('@material-ui/core/Popper'));

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
  try {
    var info = gen[key](arg);
    var value = info.value;
  } catch (error) {
    reject(error);
    return;
  }

  if (info.done) {
    resolve(value);
  } else {
    Promise.resolve(value).then(_next, _throw);
  }
}

function _asyncToGenerator(fn) {
  return function () {
    var self = this,
        args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);

      function _next(value) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
      }

      function _throw(err) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
      }

      _next(undefined);
    });
  };
}

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

var ConfirmationDialog = function ConfirmationDialog(props) {
  return React__default.createElement(core.Dialog, {
    open: props.opened,
    onClose: props.onDeny,
    "aria-labelledby": "alert-dialog-title",
    "aria-describedby": "alert-dialog-description"
  }, props.title && React__default.createElement(core.DialogTitle, {
    id: "alert-dialog-title"
  }, props.title), React__default.createElement(core.DialogContent, null, React__default.createElement(core.DialogContentText, {
    id: "alert-dialog-description"
  }, props.text)), React__default.createElement(core.DialogActions, null, React__default.createElement(core.Button, {
    onClick: props.onDeny,
    color: "primary"
  }, "Disagree"), React__default.createElement(core.Button, {
    onClick: props.onSuccess,
    color: "primary",
    autoFocus: true
  }, "Agree")));
};

var CustomDialog = function CustomDialog(props) {
  var _props$cancelationTex, _props$confirmationTe;

  return React__default.createElement(core.Dialog, {
    open: props.opened,
    onClose: props.onDeny,
    "aria-labelledby": "alert-dialog-title",
    "aria-describedby": "alert-dialog-description"
  }, props.title && React__default.createElement(core.DialogTitle, {
    id: "alert-dialog-title"
  }, props.title), React__default.createElement(core.DialogContent, null, props.text && React__default.createElement(core.DialogContentText, {
    id: "alert-dialog-description"
  }, props.text), props.children), React__default.createElement(core.DialogActions, null, React__default.createElement(core.Button, {
    onClick: props.onDeny,
    color: "primary"
  }, (_props$cancelationTex = props.cancelationText) != null ? _props$cancelationTex : "Cancel"), React__default.createElement(core.Button, {
    onClick: props.onSuccess,
    color: "primary",
    autoFocus: true
  }, (_props$confirmationTe = props.confirmationText) != null ? _props$confirmationTe : "OK")));
};

var Validator = function Validator(_ref) {
  var controlComponent = _ref.controlComponent,
      controlComponentProps = _ref.controlComponentProps,
      name = _ref.name,
      register = _ref.register,
      disabledParam = _ref.disabledParam,
      defaultErrorMessage = _ref.defaultErrorMessage,
      className = _ref.className;
  return React__default.createElement(core.Grid, {
    container: true,
    direction: "row",
    className: className,
    spacing: 1
  }, React__default.createElement(core.Grid, {
    item: true,
    xs: 6
  }, React__default.createElement(controlComponent, _extends({
    name: name,
    register: register
  }, controlComponentProps))), React__default.createElement(core.Grid, {
    item: true,
    xs: 6
  }, React__default.createElement(TextInput, {
    register: register,
    name: name + "ErrorMessage",
    type: "text",
    label: "error message",
    muiProps: {
      disabled: disabledParam,
      defaultValue: defaultErrorMessage
    }
  })));
};

var Max = function Max(props) {
  return React__default.createElement(Validator, Object.assign({}, props, {
    controlComponent: TextInput,
    controlComponentProps: {
      type: "number",
      label: "Maximal value/length"
    },
    defaultErrorMessage: "Value is too long"
  }));
};

var Min = function Min(props) {
  return React__default.createElement(Validator, Object.assign({}, props, {
    controlComponent: TextInput,
    controlComponentProps: {
      type: "number",
      label: "Minimal value/length"
    },
    defaultErrorMessage: "Value is too short"
  }));
};

var Regex = function Regex(props) {
  return React__default.createElement(Validator, Object.assign({}, props, {
    controlComponent: TextInput,
    controlComponentProps: {
      label: "Regular expression",
      muiProps: {
        InputProps: {
          startAdornment: React__default.createElement(core.InputAdornment, {
            position: "start"
          }, "/"),
          endAdornment: React__default.createElement(core.InputAdornment, {
            position: "end"
          }, "/")
        }
      }
    },
    defaultErrorMessage: "Value does not match expression"
  }));
};

var Required = function Required(props) {
  return React__default.createElement(Validator, Object.assign({}, props, {
    controlComponent: CheckBox,
    controlComponentProps: {
      options: [{
        name: "Required"
      }]
    },
    defaultErrorMessage: "Field is required"
  }));
};

var Integer = function Integer(props) {
  return React__default.createElement(Validator, Object.assign({}, props, {
    controlComponent: Switch,
    controlComponentProps: {
      label: "Integer"
    },
    defaultErrorMessage: "Value is not an integer"
  }));
};

var LessThan = function LessThan(props) {
  return React__default.createElement(Validator, Object.assign({}, props, {
    controlComponent: TextInput,
    controlComponentProps: {
      type: "number",
      label: "Less than"
    },
    defaultErrorMessage: "Value is too high"
  }));
};

var MoreThan = function MoreThan(props) {
  return React__default.createElement(Validator, Object.assign({}, props, {
    controlComponent: TextInput,
    controlComponentProps: {
      type: "number",
      label: "More than"
    },
    defaultErrorMessage: "Value is too low"
  }));
};

var Negative = function Negative(props) {
  return React__default.createElement(Validator, Object.assign({}, props, {
    controlComponent: Switch,
    controlComponentProps: {
      label: "Negative"
    },
    defaultErrorMessage: "Value is not negative"
  }));
};

var Positive = function Positive(props) {
  return React__default.createElement(Validator, Object.assign({}, props, {
    controlComponent: Switch,
    controlComponentProps: {
      label: "Positive"
    },
    defaultErrorMessage: "Value is not positive"
  }));
};

var TextInput = function TextInput(props) {
  var _props$error$message, _props$error, _props$error2;

  return React__default.createElement(FormComponentWrapper, {
    variant: "outlined"
  }, React__default.createElement(core.TextField, Object.assign({
    inputRef: props.register,
    name: props.name,
    label: props.label,
    placeholder: props.placeholder,
    type: props.type,
    helperText: (_props$error$message = (_props$error = props.error) == null ? void 0 : _props$error.message) != null ? _props$error$message : props.helperText,
    error: !!((_props$error2 = props.error) == null ? void 0 : _props$error2.message),
    InputLabelProps: {
      shrink: true
    }
  }, props.muiProps)));
};

var Length = function Length(props) {
  return React__default.createElement(Validator, Object.assign({}, props, {
    controlComponent: TextInput,
    controlComponentProps: {
      type: "number",
      label: "Length"
    },
    defaultErrorMessage: "Value must be exactly X characters long"
  }));
};

var Switch = function Switch(props) {
  var _props$error, _props$error2, _props$error3, _props$error$message, _props$error4;

  return React__default.createElement(FormComponentWrapper, {
    error: !!((_props$error = props.error) == null ? void 0 : _props$error.message)
  }, React__default.createElement(core.FormControlLabel, {
    control: React__default.createElement(core.Switch, Object.assign({
      inputRef: props.register,
      name: props.name
    }, props.muiProps)),
    label: props.label
  }), (props.helperText || ((_props$error2 = props.error) == null ? void 0 : _props$error2.message)) && React__default.createElement(core.FormHelperText, {
    error: !!((_props$error3 = props.error) == null ? void 0 : _props$error3.message)
  }, (_props$error$message = (_props$error4 = props.error) == null ? void 0 : _props$error4.message) != null ? _props$error$message : props.helperText));
};

var Email = function Email(props) {
  return React__default.createElement(Validator, Object.assign({}, props, {
    controlComponent: Switch,
    controlComponentProps: {
      label: "Email"
    },
    defaultErrorMessage: "Value is not a valid email address"
  }));
};

var Url = function Url(props) {
  return React__default.createElement(Validator, Object.assign({}, props, {
    controlComponent: Switch,
    controlComponentProps: {
      label: "URL"
    },
    defaultErrorMessage: "Value is not a valid url"
  }));
};

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var runtime_1 = createCommonjsModule(function (module) {
/**
 * Copyright (c) 2014-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

var runtime = (function (exports) {

  var Op = Object.prototype;
  var hasOwn = Op.hasOwnProperty;
  var undefined$1; // More compressible than void 0.
  var $Symbol = typeof Symbol === "function" ? Symbol : {};
  var iteratorSymbol = $Symbol.iterator || "@@iterator";
  var asyncIteratorSymbol = $Symbol.asyncIterator || "@@asyncIterator";
  var toStringTagSymbol = $Symbol.toStringTag || "@@toStringTag";

  function define(obj, key, value) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
    return obj[key];
  }
  try {
    // IE 8 has a broken Object.defineProperty that only works on DOM objects.
    define({}, "");
  } catch (err) {
    define = function(obj, key, value) {
      return obj[key] = value;
    };
  }

  function wrap(innerFn, outerFn, self, tryLocsList) {
    // If outerFn provided and outerFn.prototype is a Generator, then outerFn.prototype instanceof Generator.
    var protoGenerator = outerFn && outerFn.prototype instanceof Generator ? outerFn : Generator;
    var generator = Object.create(protoGenerator.prototype);
    var context = new Context(tryLocsList || []);

    // The ._invoke method unifies the implementations of the .next,
    // .throw, and .return methods.
    generator._invoke = makeInvokeMethod(innerFn, self, context);

    return generator;
  }
  exports.wrap = wrap;

  // Try/catch helper to minimize deoptimizations. Returns a completion
  // record like context.tryEntries[i].completion. This interface could
  // have been (and was previously) designed to take a closure to be
  // invoked without arguments, but in all the cases we care about we
  // already have an existing method we want to call, so there's no need
  // to create a new function object. We can even get away with assuming
  // the method takes exactly one argument, since that happens to be true
  // in every case, so we don't have to touch the arguments object. The
  // only additional allocation required is the completion record, which
  // has a stable shape and so hopefully should be cheap to allocate.
  function tryCatch(fn, obj, arg) {
    try {
      return { type: "normal", arg: fn.call(obj, arg) };
    } catch (err) {
      return { type: "throw", arg: err };
    }
  }

  var GenStateSuspendedStart = "suspendedStart";
  var GenStateSuspendedYield = "suspendedYield";
  var GenStateExecuting = "executing";
  var GenStateCompleted = "completed";

  // Returning this object from the innerFn has the same effect as
  // breaking out of the dispatch switch statement.
  var ContinueSentinel = {};

  // Dummy constructor functions that we use as the .constructor and
  // .constructor.prototype properties for functions that return Generator
  // objects. For full spec compliance, you may wish to configure your
  // minifier not to mangle the names of these two functions.
  function Generator() {}
  function GeneratorFunction() {}
  function GeneratorFunctionPrototype() {}

  // This is a polyfill for %IteratorPrototype% for environments that
  // don't natively support it.
  var IteratorPrototype = {};
  IteratorPrototype[iteratorSymbol] = function () {
    return this;
  };

  var getProto = Object.getPrototypeOf;
  var NativeIteratorPrototype = getProto && getProto(getProto(values([])));
  if (NativeIteratorPrototype &&
      NativeIteratorPrototype !== Op &&
      hasOwn.call(NativeIteratorPrototype, iteratorSymbol)) {
    // This environment has a native %IteratorPrototype%; use it instead
    // of the polyfill.
    IteratorPrototype = NativeIteratorPrototype;
  }

  var Gp = GeneratorFunctionPrototype.prototype =
    Generator.prototype = Object.create(IteratorPrototype);
  GeneratorFunction.prototype = Gp.constructor = GeneratorFunctionPrototype;
  GeneratorFunctionPrototype.constructor = GeneratorFunction;
  GeneratorFunction.displayName = define(
    GeneratorFunctionPrototype,
    toStringTagSymbol,
    "GeneratorFunction"
  );

  // Helper for defining the .next, .throw, and .return methods of the
  // Iterator interface in terms of a single ._invoke method.
  function defineIteratorMethods(prototype) {
    ["next", "throw", "return"].forEach(function(method) {
      define(prototype, method, function(arg) {
        return this._invoke(method, arg);
      });
    });
  }

  exports.isGeneratorFunction = function(genFun) {
    var ctor = typeof genFun === "function" && genFun.constructor;
    return ctor
      ? ctor === GeneratorFunction ||
        // For the native GeneratorFunction constructor, the best we can
        // do is to check its .name property.
        (ctor.displayName || ctor.name) === "GeneratorFunction"
      : false;
  };

  exports.mark = function(genFun) {
    if (Object.setPrototypeOf) {
      Object.setPrototypeOf(genFun, GeneratorFunctionPrototype);
    } else {
      genFun.__proto__ = GeneratorFunctionPrototype;
      define(genFun, toStringTagSymbol, "GeneratorFunction");
    }
    genFun.prototype = Object.create(Gp);
    return genFun;
  };

  // Within the body of any async function, `await x` is transformed to
  // `yield regeneratorRuntime.awrap(x)`, so that the runtime can test
  // `hasOwn.call(value, "__await")` to determine if the yielded value is
  // meant to be awaited.
  exports.awrap = function(arg) {
    return { __await: arg };
  };

  function AsyncIterator(generator, PromiseImpl) {
    function invoke(method, arg, resolve, reject) {
      var record = tryCatch(generator[method], generator, arg);
      if (record.type === "throw") {
        reject(record.arg);
      } else {
        var result = record.arg;
        var value = result.value;
        if (value &&
            typeof value === "object" &&
            hasOwn.call(value, "__await")) {
          return PromiseImpl.resolve(value.__await).then(function(value) {
            invoke("next", value, resolve, reject);
          }, function(err) {
            invoke("throw", err, resolve, reject);
          });
        }

        return PromiseImpl.resolve(value).then(function(unwrapped) {
          // When a yielded Promise is resolved, its final value becomes
          // the .value of the Promise<{value,done}> result for the
          // current iteration.
          result.value = unwrapped;
          resolve(result);
        }, function(error) {
          // If a rejected Promise was yielded, throw the rejection back
          // into the async generator function so it can be handled there.
          return invoke("throw", error, resolve, reject);
        });
      }
    }

    var previousPromise;

    function enqueue(method, arg) {
      function callInvokeWithMethodAndArg() {
        return new PromiseImpl(function(resolve, reject) {
          invoke(method, arg, resolve, reject);
        });
      }

      return previousPromise =
        // If enqueue has been called before, then we want to wait until
        // all previous Promises have been resolved before calling invoke,
        // so that results are always delivered in the correct order. If
        // enqueue has not been called before, then it is important to
        // call invoke immediately, without waiting on a callback to fire,
        // so that the async generator function has the opportunity to do
        // any necessary setup in a predictable way. This predictability
        // is why the Promise constructor synchronously invokes its
        // executor callback, and why async functions synchronously
        // execute code before the first await. Since we implement simple
        // async functions in terms of async generators, it is especially
        // important to get this right, even though it requires care.
        previousPromise ? previousPromise.then(
          callInvokeWithMethodAndArg,
          // Avoid propagating failures to Promises returned by later
          // invocations of the iterator.
          callInvokeWithMethodAndArg
        ) : callInvokeWithMethodAndArg();
    }

    // Define the unified helper method that is used to implement .next,
    // .throw, and .return (see defineIteratorMethods).
    this._invoke = enqueue;
  }

  defineIteratorMethods(AsyncIterator.prototype);
  AsyncIterator.prototype[asyncIteratorSymbol] = function () {
    return this;
  };
  exports.AsyncIterator = AsyncIterator;

  // Note that simple async functions are implemented on top of
  // AsyncIterator objects; they just return a Promise for the value of
  // the final result produced by the iterator.
  exports.async = function(innerFn, outerFn, self, tryLocsList, PromiseImpl) {
    if (PromiseImpl === void 0) PromiseImpl = Promise;

    var iter = new AsyncIterator(
      wrap(innerFn, outerFn, self, tryLocsList),
      PromiseImpl
    );

    return exports.isGeneratorFunction(outerFn)
      ? iter // If outerFn is a generator, return the full iterator.
      : iter.next().then(function(result) {
          return result.done ? result.value : iter.next();
        });
  };

  function makeInvokeMethod(innerFn, self, context) {
    var state = GenStateSuspendedStart;

    return function invoke(method, arg) {
      if (state === GenStateExecuting) {
        throw new Error("Generator is already running");
      }

      if (state === GenStateCompleted) {
        if (method === "throw") {
          throw arg;
        }

        // Be forgiving, per 25.3.3.3.3 of the spec:
        // https://people.mozilla.org/~jorendorff/es6-draft.html#sec-generatorresume
        return doneResult();
      }

      context.method = method;
      context.arg = arg;

      while (true) {
        var delegate = context.delegate;
        if (delegate) {
          var delegateResult = maybeInvokeDelegate(delegate, context);
          if (delegateResult) {
            if (delegateResult === ContinueSentinel) continue;
            return delegateResult;
          }
        }

        if (context.method === "next") {
          // Setting context._sent for legacy support of Babel's
          // function.sent implementation.
          context.sent = context._sent = context.arg;

        } else if (context.method === "throw") {
          if (state === GenStateSuspendedStart) {
            state = GenStateCompleted;
            throw context.arg;
          }

          context.dispatchException(context.arg);

        } else if (context.method === "return") {
          context.abrupt("return", context.arg);
        }

        state = GenStateExecuting;

        var record = tryCatch(innerFn, self, context);
        if (record.type === "normal") {
          // If an exception is thrown from innerFn, we leave state ===
          // GenStateExecuting and loop back for another invocation.
          state = context.done
            ? GenStateCompleted
            : GenStateSuspendedYield;

          if (record.arg === ContinueSentinel) {
            continue;
          }

          return {
            value: record.arg,
            done: context.done
          };

        } else if (record.type === "throw") {
          state = GenStateCompleted;
          // Dispatch the exception by looping back around to the
          // context.dispatchException(context.arg) call above.
          context.method = "throw";
          context.arg = record.arg;
        }
      }
    };
  }

  // Call delegate.iterator[context.method](context.arg) and handle the
  // result, either by returning a { value, done } result from the
  // delegate iterator, or by modifying context.method and context.arg,
  // setting context.delegate to null, and returning the ContinueSentinel.
  function maybeInvokeDelegate(delegate, context) {
    var method = delegate.iterator[context.method];
    if (method === undefined$1) {
      // A .throw or .return when the delegate iterator has no .throw
      // method always terminates the yield* loop.
      context.delegate = null;

      if (context.method === "throw") {
        // Note: ["return"] must be used for ES3 parsing compatibility.
        if (delegate.iterator["return"]) {
          // If the delegate iterator has a return method, give it a
          // chance to clean up.
          context.method = "return";
          context.arg = undefined$1;
          maybeInvokeDelegate(delegate, context);

          if (context.method === "throw") {
            // If maybeInvokeDelegate(context) changed context.method from
            // "return" to "throw", let that override the TypeError below.
            return ContinueSentinel;
          }
        }

        context.method = "throw";
        context.arg = new TypeError(
          "The iterator does not provide a 'throw' method");
      }

      return ContinueSentinel;
    }

    var record = tryCatch(method, delegate.iterator, context.arg);

    if (record.type === "throw") {
      context.method = "throw";
      context.arg = record.arg;
      context.delegate = null;
      return ContinueSentinel;
    }

    var info = record.arg;

    if (! info) {
      context.method = "throw";
      context.arg = new TypeError("iterator result is not an object");
      context.delegate = null;
      return ContinueSentinel;
    }

    if (info.done) {
      // Assign the result of the finished delegate to the temporary
      // variable specified by delegate.resultName (see delegateYield).
      context[delegate.resultName] = info.value;

      // Resume execution at the desired location (see delegateYield).
      context.next = delegate.nextLoc;

      // If context.method was "throw" but the delegate handled the
      // exception, let the outer generator proceed normally. If
      // context.method was "next", forget context.arg since it has been
      // "consumed" by the delegate iterator. If context.method was
      // "return", allow the original .return call to continue in the
      // outer generator.
      if (context.method !== "return") {
        context.method = "next";
        context.arg = undefined$1;
      }

    } else {
      // Re-yield the result returned by the delegate method.
      return info;
    }

    // The delegate iterator is finished, so forget it and continue with
    // the outer generator.
    context.delegate = null;
    return ContinueSentinel;
  }

  // Define Generator.prototype.{next,throw,return} in terms of the
  // unified ._invoke helper method.
  defineIteratorMethods(Gp);

  define(Gp, toStringTagSymbol, "Generator");

  // A Generator should always return itself as the iterator object when the
  // @@iterator function is called on it. Some browsers' implementations of the
  // iterator prototype chain incorrectly implement this, causing the Generator
  // object to not be returned from this call. This ensures that doesn't happen.
  // See https://github.com/facebook/regenerator/issues/274 for more details.
  Gp[iteratorSymbol] = function() {
    return this;
  };

  Gp.toString = function() {
    return "[object Generator]";
  };

  function pushTryEntry(locs) {
    var entry = { tryLoc: locs[0] };

    if (1 in locs) {
      entry.catchLoc = locs[1];
    }

    if (2 in locs) {
      entry.finallyLoc = locs[2];
      entry.afterLoc = locs[3];
    }

    this.tryEntries.push(entry);
  }

  function resetTryEntry(entry) {
    var record = entry.completion || {};
    record.type = "normal";
    delete record.arg;
    entry.completion = record;
  }

  function Context(tryLocsList) {
    // The root entry object (effectively a try statement without a catch
    // or a finally block) gives us a place to store values thrown from
    // locations where there is no enclosing try statement.
    this.tryEntries = [{ tryLoc: "root" }];
    tryLocsList.forEach(pushTryEntry, this);
    this.reset(true);
  }

  exports.keys = function(object) {
    var keys = [];
    for (var key in object) {
      keys.push(key);
    }
    keys.reverse();

    // Rather than returning an object with a next method, we keep
    // things simple and return the next function itself.
    return function next() {
      while (keys.length) {
        var key = keys.pop();
        if (key in object) {
          next.value = key;
          next.done = false;
          return next;
        }
      }

      // To avoid creating an additional object, we just hang the .value
      // and .done properties off the next function object itself. This
      // also ensures that the minifier will not anonymize the function.
      next.done = true;
      return next;
    };
  };

  function values(iterable) {
    if (iterable) {
      var iteratorMethod = iterable[iteratorSymbol];
      if (iteratorMethod) {
        return iteratorMethod.call(iterable);
      }

      if (typeof iterable.next === "function") {
        return iterable;
      }

      if (!isNaN(iterable.length)) {
        var i = -1, next = function next() {
          while (++i < iterable.length) {
            if (hasOwn.call(iterable, i)) {
              next.value = iterable[i];
              next.done = false;
              return next;
            }
          }

          next.value = undefined$1;
          next.done = true;

          return next;
        };

        return next.next = next;
      }
    }

    // Return an iterator with no values.
    return { next: doneResult };
  }
  exports.values = values;

  function doneResult() {
    return { value: undefined$1, done: true };
  }

  Context.prototype = {
    constructor: Context,

    reset: function(skipTempReset) {
      this.prev = 0;
      this.next = 0;
      // Resetting context._sent for legacy support of Babel's
      // function.sent implementation.
      this.sent = this._sent = undefined$1;
      this.done = false;
      this.delegate = null;

      this.method = "next";
      this.arg = undefined$1;

      this.tryEntries.forEach(resetTryEntry);

      if (!skipTempReset) {
        for (var name in this) {
          // Not sure about the optimal order of these conditions:
          if (name.charAt(0) === "t" &&
              hasOwn.call(this, name) &&
              !isNaN(+name.slice(1))) {
            this[name] = undefined$1;
          }
        }
      }
    },

    stop: function() {
      this.done = true;

      var rootEntry = this.tryEntries[0];
      var rootRecord = rootEntry.completion;
      if (rootRecord.type === "throw") {
        throw rootRecord.arg;
      }

      return this.rval;
    },

    dispatchException: function(exception) {
      if (this.done) {
        throw exception;
      }

      var context = this;
      function handle(loc, caught) {
        record.type = "throw";
        record.arg = exception;
        context.next = loc;

        if (caught) {
          // If the dispatched exception was caught by a catch block,
          // then let that catch block handle the exception normally.
          context.method = "next";
          context.arg = undefined$1;
        }

        return !! caught;
      }

      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];
        var record = entry.completion;

        if (entry.tryLoc === "root") {
          // Exception thrown outside of any try block that could handle
          // it, so set the completion value of the entire function to
          // throw the exception.
          return handle("end");
        }

        if (entry.tryLoc <= this.prev) {
          var hasCatch = hasOwn.call(entry, "catchLoc");
          var hasFinally = hasOwn.call(entry, "finallyLoc");

          if (hasCatch && hasFinally) {
            if (this.prev < entry.catchLoc) {
              return handle(entry.catchLoc, true);
            } else if (this.prev < entry.finallyLoc) {
              return handle(entry.finallyLoc);
            }

          } else if (hasCatch) {
            if (this.prev < entry.catchLoc) {
              return handle(entry.catchLoc, true);
            }

          } else if (hasFinally) {
            if (this.prev < entry.finallyLoc) {
              return handle(entry.finallyLoc);
            }

          } else {
            throw new Error("try statement without catch or finally");
          }
        }
      }
    },

    abrupt: function(type, arg) {
      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];
        if (entry.tryLoc <= this.prev &&
            hasOwn.call(entry, "finallyLoc") &&
            this.prev < entry.finallyLoc) {
          var finallyEntry = entry;
          break;
        }
      }

      if (finallyEntry &&
          (type === "break" ||
           type === "continue") &&
          finallyEntry.tryLoc <= arg &&
          arg <= finallyEntry.finallyLoc) {
        // Ignore the finally entry if control is not jumping to a
        // location outside the try/catch block.
        finallyEntry = null;
      }

      var record = finallyEntry ? finallyEntry.completion : {};
      record.type = type;
      record.arg = arg;

      if (finallyEntry) {
        this.method = "next";
        this.next = finallyEntry.finallyLoc;
        return ContinueSentinel;
      }

      return this.complete(record);
    },

    complete: function(record, afterLoc) {
      if (record.type === "throw") {
        throw record.arg;
      }

      if (record.type === "break" ||
          record.type === "continue") {
        this.next = record.arg;
      } else if (record.type === "return") {
        this.rval = this.arg = record.arg;
        this.method = "return";
        this.next = "end";
      } else if (record.type === "normal" && afterLoc) {
        this.next = afterLoc;
      }

      return ContinueSentinel;
    },

    finish: function(finallyLoc) {
      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];
        if (entry.finallyLoc === finallyLoc) {
          this.complete(entry.completion, entry.afterLoc);
          resetTryEntry(entry);
          return ContinueSentinel;
        }
      }
    },

    "catch": function(tryLoc) {
      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];
        if (entry.tryLoc === tryLoc) {
          var record = entry.completion;
          if (record.type === "throw") {
            var thrown = record.arg;
            resetTryEntry(entry);
          }
          return thrown;
        }
      }

      // The context.catch method must only be called with a location
      // argument that corresponds to a known catch block.
      throw new Error("illegal catch attempt");
    },

    delegateYield: function(iterable, resultName, nextLoc) {
      this.delegate = {
        iterator: values(iterable),
        resultName: resultName,
        nextLoc: nextLoc
      };

      if (this.method === "next") {
        // Deliberately forget the last sent value so that we don't
        // accidentally pass it on to the delegate.
        this.arg = undefined$1;
      }

      return ContinueSentinel;
    }
  };

  // Regardless of whether this script is executing as a CommonJS module
  // or not, return the runtime object so that we can declare the variable
  // regeneratorRuntime in the outer scope, which allows this module to be
  // injected easily by `bin/regenerator --include-runtime script.js`.
  return exports;

}(
  // If this script is executing as a CommonJS module, use module.exports
  // as the regeneratorRuntime namespace. Otherwise create a new empty
  // object. Either way, the resulting object will be used to initialize
  // the regeneratorRuntime variable at the top of this file.
   module.exports 
));

try {
  regeneratorRuntime = runtime;
} catch (accidentalStrictMode) {
  // This module should not be running in strict mode, so the above
  // assignment should always work unless something is misconfigured. Just
  // in case runtime.js accidentally runs in strict mode, we can escape
  // strict mode using a global Function call. This could conceivably fail
  // if a Content Security Policy forbids using Function, but in that case
  // the proper solution is to fix the accidental strict mode problem. If
  // you've misconfigured your bundler to force strict mode and applied a
  // CSP to forbid Function, and you're not willing to fix either of those
  // problems, please detail your unique predicament in a GitHub issue.
  Function("r", "regeneratorRuntime = r")(runtime);
}
});

function uuid() {
  var i, random;
  var uuid = "";

  for (i = 0; i < 32; i++) {
    random = Math.random() * 16 | 0;

    if (i === 8 || i === 12 || i === 16 || i === 20) {
      uuid += "-";
    }

    uuid += (i === 12 ? 4 : i === 16 ? random & 3 | 8 : random).toString(16);
  }

  return uuid;
}

var formElementActions = {
  addFormElement: function addFormElement(store, element) {
    store.setState({
      elements: [].concat(store.state.elements, [element])
    });
  },
  addFormElementAtIndex: function addFormElementAtIndex(store, payload, index) {
    var elements = [].concat(store.state.elements);
    elements.splice(index, 0, payload);
    store.setState({
      elements: elements
    });
  },
  removeFormElement: function removeFormElement(store, id) {
    store.setState({
      elements: store.state.elements.filter(function (el) {
        return el.id !== id;
      })
    });
  },
  copyFormElement: function copyFormElement(store, id, name, position) {
    var duplicateElement = store.state.elements.find(function (el) {
      return el.id === id;
    });
    duplicateElement.name = name;
    var index = store.state.elements.findIndex(function (el) {
      return el.id === id;
    });

    if (!duplicateElement) {
      return null;
    }

    var newElement = _extends({}, duplicateElement, {
      id: uuid(),
      groupPosition: position
    });

    formElementActions.addFormElementAtIndex(store, newElement, index + 1);
    return null;
  },
  swapFormElements: function swapFormElements(store, groupID, oldIndex, newIndex) {
    var elements = store.state.elements.filter(function (el) {
      return el.groupID === groupID;
    });
    var restElements = store.state.elements.filter(function (el) {
      return el.groupID !== groupID;
    });
    var elementA = elements[oldIndex];
    var elementB = elements[newIndex]; // swap group position indexes

    var index = elementA.groupPosition;
    elementA.groupPosition = elementB.groupPosition;
    elementB.groupPosition = index;
    elements.splice(oldIndex, 1, elementA);
    elements.splice(newIndex, 1, elementB);
    store.setState({
      elements: [].concat(elements.sort(function (a, b) {
        return a.groupPosition - b.groupPosition;
      }), restElements)
    });
  },
  transferElement: function transferElement(store, id, groupId) {
    formElementActions.setFormElementAttribute(store, id, 'groupID', groupId);
  },
  setFormElementAttributes: function setFormElementAttributes(store, id, attrs) {
    var elements = [].concat(store.state.elements);
    var element = elements.find(function (el) {
      return el.id === id;
    });
    var elementId = elements.findIndex(function (el) {
      return el.id === id;
    });

    if (!element) {
      return null;
    }

    var updatedElement = _extends({}, element, {
      attributes: _extends({}, attrs)
    });

    elements.splice(elementId, 1, updatedElement);
    store.setState({
      elements: elements
    });
    return null;
  },
  setFormElementValidations: function setFormElementValidations(store, id, validations) {
    var elements = [].concat(store.state.elements);
    var element = elements.find(function (el) {
      return el.id === id;
    });
    var elementId = elements.findIndex(function (el) {
      return el.id === id;
    });

    if (!element) {
      return null;
    }

    var updatedElement = _extends({}, element, {
      attributes: _extends({}, element.attributes, {
        required: validations.required
      }),
      validations: _extends({}, validations)
    });

    elements.splice(elementId, 1, updatedElement);
    store.setState({
      elements: elements
    });
    return null;
  },
  setFormElementAttribute: function setFormElementAttribute(store, id, attribute, value) {
    var _extends2;

    var elements = [].concat(store.state.elements);
    var element = elements.find(function (el) {
      return el.id === id;
    });
    var elementId = elements.findIndex(function (el) {
      return el.id === id;
    });

    if (!element) {
      return null;
    }

    var updatedElement = _extends({}, element, (_extends2 = {}, _extends2[attribute] = value, _extends2));

    elements.splice(elementId, 1, updatedElement);
    store.setState({
      elements: elements
    });
    return null;
  }
};

var groupingActions = {
  addGroup: function addGroup(store, group) {
    store.setState({
      grouping: {
        activeGroup: group.id,
        groups: [].concat(store.state.grouping.groups, [group])
      }
    });
  },
  setActiveGroup: function setActiveGroup(store, id) {
    store.setState({
      grouping: {
        groups: [].concat(store.state.grouping.groups),
        activeGroup: id
      }
    });
  },
  removeGroup: function removeGroup(store, id) {
    var _newActiveGroup$id;

    if (store.state.grouping.groups.length === 1) {
      return null;
    }

    var newActiveGroup = store.state.grouping.groups.find(function (g) {
      return g.id !== id;
    });
    store.setState({
      grouping: {
        groups: [].concat(store.state.grouping.groups.filter(function (g) {
          return g.id !== id;
        })),
        activeGroup: (_newActiveGroup$id = newActiveGroup == null ? void 0 : newActiveGroup.id) != null ? _newActiveGroup$id : store.state.grouping.groups[0].id
      },
      elements: [].concat(store.state.elements.filter(function (el) {
        return el.groupID !== id;
      }))
    });
    return null;
  },
  renameGroup: function renameGroup(store, id, name) {
    var groups = [].concat(store.state.grouping.groups);
    var group = groups.find(function (g) {
      return g.id === id;
    });
    var groupIndex = groups.findIndex(function (g) {
      return g.id === id;
    });
    console.log({
      group: group,
      groupIndex: groupIndex
    });

    if (!group || groupIndex === -1) {
      return null;
    }

    var updatedGroup = _extends({}, group, {
      name: name
    });

    groups.splice(groupIndex, 1, updatedGroup);
    store.setState({
      grouping: {
        groups: groups,
        activeGroup: store.state.grouping.activeGroup
      }
    });
    return null;
  },
  swapGroups: function swapGroups(store, oldIndex, newIndex) {
    var groups = [].concat(store.state.grouping.groups);
    groups[oldIndex] = groups.splice(newIndex, 1, groups[oldIndex])[0];
    store.setState({
      grouping: {
        groups: groups,
        activeGroup: store.state.grouping.activeGroup
      }
    });
  }
};

var storeActions = {
  setStore: function setStore(store, newStore) {
    store.setState({
      elements: newStore.elements,
      grouping: newStore.grouping
    });
  },
  clearStore: function clearStore(store) {
    store.setState(initialState);
  }
};

var defaultGroupID = /*#__PURE__*/uuid();
var initialState = {
  elements: [],
  grouping: {
    groups: [{
      id: defaultGroupID,
      name: 'default group'
    }],
    activeGroup: defaultGroupID
  }
};

var _createStore = /*#__PURE__*/reactSimpleHookStore.createStore(initialState, {
  formElementsActions: formElementActions,
  groupingActions: groupingActions,
  storeActions: storeActions
}),
    useStore = _createStore.useStore;

var useDataExporter = function useDataExporter() {
  var _useStore = useStore(function (s) {
    return s;
  }, function (a) {
    return a.storeActions;
  }),
      store = _useStore[0],
      storeActions = _useStore[1];

  var _useRfaDataConverter = useRfaDataConverter(),
      _useRfaDataConverter$ = _useRfaDataConverter.handlers,
      dataToStore = _useRfaDataConverter$.dataToStore,
      dataToExport = _useRfaDataConverter$.dataToExport;

  var _React$useState = React__default.useState(false),
      copied = _React$useState[0],
      setCopied = _React$useState[1];

  var _React$useState2 = React__default.useState(false),
      exported = _React$useState2[0],
      setExported = _React$useState2[1];

  var _React$useState3 = React__default.useState(false),
      imported = _React$useState3[0],
      setImported = _React$useState3[1];

  var getStringData = React__default.useCallback(function () {
    return JSON.stringify(store);
  }, [store]);
  var saveAsJSON = React__default.useCallback(function () {
    var storeToExport = dataToExport(store);
    var stringData = JSON.stringify(storeToExport);
    var out = [];

    for (var i = 0; i < stringData.length; i++) {
      out[i] = stringData.charCodeAt(i);
    }

    var data = new Uint8Array(out);
    var blob = new Blob([data], {
      type: 'application/octet-stream'
    });
    var url = URL.createObjectURL(blob);
    var link = document.createElement('a');
    link.setAttribute('href', url);
    link.setAttribute('download', 'small_step_for_rfa.json');
    link.click();
    setExported(true);
  }, [getStringData]);

  var isTypeofState = function isTypeofState(data) {
    return !!(data.elements && data.elements.length && data.elements[0].id && data.elements[0].name && data.elements[0].groupID && data.grouping);
  };

  var importData = React__default.useCallback(function (jsonData) {
    if (!isTypeofState(jsonData)) {
      alert('import unsuccessful, wrong type!');
      setImported(false);
      return;
    }

    var storeData = dataToStore(jsonData);
    storeActions.clearStore();
    storeActions.setStore(storeData);
    setImported(true);
  }, [storeActions, dataToStore]);
  var copyToClipboard = React__default.useCallback( /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee() {
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.prev = 0;
            _context.next = 3;
            return navigator.clipboard.writeText(getStringData());

          case 3:
            setCopied(true);
            _context.next = 10;
            break;

          case 6:
            _context.prev = 6;
            _context.t0 = _context["catch"](0);
            console.log(_context.t0);
            setCopied(false);

          case 10:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, null, [[0, 6]]);
  })), [getStringData]);
  return {
    handlers: {
      saveAsJSON: saveAsJSON,
      importData: importData,
      copyToClipboard: copyToClipboard
    },
    state: {
      copied: copied,
      exported: exported,
      imported: imported
    }
  };
};

var useDialog = function useDialog(init) {
  if (init === void 0) {
    init = false;
  }

  var _useState = React.useState(init),
      open = _useState[0],
      setOpen = _useState[1];

  var handleOpen = React.useCallback(function () {
    return setOpen(true);
  }, []);
  var handleClose = React.useCallback(function () {
    return setOpen(false);
  }, []);
  return [open, handleOpen, handleClose];
};

var HelperTooltip = function HelperTooltip(_ref) {
  var text = _ref.text;
  return React__default.createElement(core.Tooltip, {
    title: text
  }, React__default.createElement(icons.HelpOutline, {
    fontSize: "small",
    color: "action"
  }));
};

var TreeNodeDataBuilder = function TreeNodeDataBuilder(props) {
  var classes = useStyles();

  var _useFieldArray = reactHookForm.useFieldArray({
    name: props.name,
    control: props.control
  }),
      fields = _useFieldArray.fields,
      append = _useFieldArray.append,
      remove = _useFieldArray.remove,
      swap = _useFieldArray.swap;

  return React.createElement(React.Fragment, null, React.createElement(core.Grid, {
    container: true,
    spacing: 1,
    direction: 'column',
    className: classes.wrapper
  }, React.createElement(core.Typography, {
    color: 'textSecondary',
    variant: 'subtitle2'
  }, "Node data"), React.createElement(core.Grid, {
    item: true,
    container: true,
    direction: 'row',
    alignItems: 'center',
    justify: 'center',
    spacing: 2,
    className: classes.container
  }, React.createElement("div", {
    style: {
      width: 26
    }
  }), React.createElement(core.Grid, {
    item: true,
    xs: 4
  }, React.createElement(core.Typography, {
    variant: 'caption',
    color: 'primary'
  }, "title")), React.createElement(core.Grid, {
    item: true,
    xs: 3
  }, React.createElement(core.Typography, {
    variant: 'caption',
    color: 'primary'
  }, "input placeholder")), React.createElement(core.Grid, {
    item: true,
    xs: 3
  }, React.createElement(core.Typography, {
    variant: 'caption',
    color: 'primary'
  }, "input value"), React.createElement(HelperTooltip, {
    text: 'When filled, user cannot edit this field, use for constants'
  })), React.createElement(core.Grid, {
    item: true,
    xs: 1
  })), React.createElement(core.Divider, null), fields.map(function (item, index) {
    return React.createElement(SortableWrapper, {
      id: item.id,
      index: index,
      swap: swap,
      dndIndicator: 'optionItemSortable',
      key: item.id
    }, React.createElement(core.Grid, {
      item: true,
      container: true,
      direction: 'row',
      alignItems: 'center',
      justify: 'center',
      spacing: 2,
      className: classes.container
    }, React.createElement("div", {
      className: classes.dragContainer
    }, React.createElement(icons.DragIndicatorOutlined, {
      color: 'action'
    })), React.createElement(core.Grid, {
      item: true,
      xs: 4
    }, React.createElement(core.Input, {
      fullWidth: true,
      placeholder: "Name " + (index + 1),
      autoFocus: true,
      onFocus: function onFocus(event) {
        event.target.select();
      },
      inputRef: props.register(),
      defaultValue: item.name,
      name: props.name + "[" + index + "].name",
      endAdornment: ':'
    })), React.createElement(core.Grid, {
      item: true,
      xs: 3
    }, React.createElement(core.Input, {
      fullWidth: true,
      placeholder: "Placeholder " + (index + 1),
      autoFocus: true,
      onFocus: function onFocus(event) {
        event.target.select();
      },
      inputRef: props.register(),
      defaultValue: item.placeholder,
      name: props.name + "[" + index + "].placeholder"
    })), React.createElement(core.Grid, {
      item: true,
      xs: 3
    }, React.createElement(core.Input, {
      fullWidth: true,
      placeholder: "Value " + (index + 1),
      autoFocus: true,
      onFocus: function onFocus(event) {
        event.target.select();
      },
      inputRef: props.register(),
      defaultValue: item.value,
      name: props.name + "[" + index + "].value"
    })), React.createElement(core.Grid, {
      item: true,
      xs: 1
    }, React.createElement(core.Tooltip, {
      title: "Remove option",
      "aria-label": "remove-option",
      enterDelay: 500
    }, React.createElement(core.IconButton, {
      "aria-label": "delete",
      onClick: function onClick() {
        return remove(index);
      },
      size: 'small'
    }, React.createElement(icons.Clear, {
      color: 'error',
      fontSize: 'small'
    }))))));
  }), React.createElement(core.Box, {
    p: 2,
    className: classes.actionArea
  }, React.createElement(core.Button, {
    color: 'primary',
    variant: 'outlined',
    onClick: function onClick() {
      return append({
        name: ''
      });
    }
  }, "Add"))));
};
var useStyles = /*#__PURE__*/core.makeStyles(function (theme) {
  return core.createStyles({
    container: {
      width: '100%',
      flexWrap: 'nowrap',
      '&:hover $dragContainer': {
        visibility: 'visible'
      }
    },
    dragContainer: {
      visibility: 'hidden',
      cursor: 'move',
      position: 'relative',
      display: 'flex',
      alignItems: 'center',
      height: '100%',
      left: theme.spacing(0.5)
    },
    wrapper: {
      borderWidth: 1,
      borderStyle: 'solid',
      borderColor: theme.palette.grey.A100,
      borderRadius: theme.shape.borderRadius,
      padding: theme.spacing(2)
    },
    actionArea: {
      display: 'flex',
      justifyContent: 'center',
      alignItems: 'center'
    },
    actionDivider: {
      paddingLeft: theme.spacing(1),
      paddingRight: theme.spacing(1)
    }
  });
});

var useFieldCreator = function useFieldCreator() {
  var getAttributeEditField = function getAttributeEditField(attribute, formContextRegister, formContextControl) {
    var attributesFieldMap = {
      input: React__default.createElement(TextInput, Object.assign({}, attribute, {
        register: formContextRegister.register
      })),
      select: React__default.createElement(Select, Object.assign({}, attribute, {
        options: attribute.options,
        control: formContextControl
      })),
      checkbox: React__default.createElement("div", null, "Checkbox is not mapped yet"),
      options: React__default.createElement(OptionsBuilder, {
        name: attribute.name,
        control: formContextControl,
        register: formContextRegister.register
      }),
      radio: React__default.createElement("div", null, "radio is not mapped yet"),
      "switch": React__default.createElement(Switch, Object.assign({}, attribute, {
        register: formContextRegister.register
      })),
      keyValueData: React__default.createElement(TreeNodeDataBuilder, {
        control: formContextControl,
        name: attribute.name,
        register: formContextRegister.register
      })
    };
    return attributesFieldMap[attribute.type];
  };

  return {
    getAttributeEditField: getAttributeEditField
  };
};

var useNameGenerator = function useNameGenerator() {
  var _useStore = useStore(function (s) {
    return s.elements;
  }),
      elements = _useStore[0];

  var isUniqueName = React__default.useCallback(function (name) {
    var names = elements.map(function (el) {
      return el.name;
    });
    return !names.includes(name);
  }, [elements]);
  var createName = React__default.useCallback(function (label, prefix) {
    var cleanName = label.trim().replace(/\s+/g, '_').toLowerCase();
    var uniqueName = isUniqueName(cleanName) ? cleanName : cleanName + "-" + uuid();
    return prefix ? prefix + "-" + uniqueName : uniqueName;
  }, [isUniqueName]);
  var createNameById = React__default.useCallback(function (id) {
    var element = elements.find(function (el) {
      return el.id === id;
    });

    if (!element) {
      return '';
    }

    return createName(element.label);
  }, [createName, elements]);
  return {
    createName: createName,
    createNameById: createNameById
  };
};

var useFormElements = function useFormElements() {
  var _useStore = useStore(function (s) {
    return s.elements;
  }, function (a) {
    return a.formElementsActions;
  }),
      elements = _useStore[0],
      actions = _useStore[1];

  var _useStore2 = useStore(function (s) {
    return s.grouping.activeGroup;
  }, function () {
    return null;
  }),
      activeGroupId = _useStore2[0];

  var _useNameGenerator = useNameGenerator(),
      createName = _useNameGenerator.createName;

  var groupReducedElements = React__default.useMemo(function () {
    return elements.filter(function (el) {
      return el.groupID === activeGroupId;
    });
  }, [elements, activeGroupId]);
  var lastGroupPositionIndex = React__default.useMemo(function () {
    return groupReducedElements.length > 0 ? Math.max.apply(Math, groupReducedElements.map(function (el) {
      return el.groupPosition;
    })) : 0;
  }, [groupReducedElements]);
  var createFormElement = React__default.useCallback(function (formElement) {
    var id = uuid();
    var attrs = formElement.attributes.reduce(function (accumulator, attrObj) {
      var _extends4;

      if (attrObj.options && attrObj.type === 'options') {
        var _extends2;

        return _extends({}, accumulator, (_extends2 = {}, _extends2[attrObj.name] = attrObj.options, _extends2));
      }

      if (attrObj.keyValueData && attrObj.type === 'keyValueData') {
        var _extends3;

        return _extends({}, accumulator, (_extends3 = {}, _extends3[attrObj.name] = attrObj.keyValueData, _extends3));
      }

      return _extends({}, accumulator, (_extends4 = {}, _extends4[attrObj.name] = attrObj.value, _extends4));
    }, {});
    actions.addFormElement({
      id: id,
      label: formElement.label,
      name: createName(formElement.label),
      attributeSchema: formElement.attributes,
      attributes: attrs,
      render: formElement.render,
      isCustomRegistered: formElement.isCustomRegistered,
      groupID: activeGroupId,
      groupPosition: lastGroupPositionIndex + 1,
      isReadOnly: formElement.isReadOnly,
      validations: {},
      validationType: formElement.validationType,
      validators: formElement.validators
    });
  }, [actions, activeGroupId, createName, lastGroupPositionIndex]);
  var setFormElementAttributes = React__default.useCallback(function () {
    actions.setFormElementAttributes.apply(actions, arguments);
  }, [actions]);
  var setFormElementAttribute = React__default.useCallback(function () {
    actions.setFormElementAttribute.apply(actions, arguments);
  }, [actions]);
  var setFormElementValidations = React__default.useCallback(function () {
    actions.setFormElementValidations.apply(actions, arguments);
  }, [actions]);
  return {
    elements: groupReducedElements,
    allElements: elements,
    createFormElement: createFormElement,
    setFormElementAttributes: setFormElementAttributes,
    setFormElementAttribute: setFormElementAttribute,
    handlers: {
      setFormElementValidations: setFormElementValidations
    },
    state: {
      lastGroupPositionIndex: lastGroupPositionIndex
    }
  };
};

var useFormElementActions = function useFormElementActions(formElementId) {
  var _useStore = useStore(function () {
    return null;
  }, function (a) {
    return a.formElementsActions;
  }),
      actions = _useStore[1];

  var _useFormElements = useFormElements(),
      lastGroupPositionIndex = _useFormElements.state.lastGroupPositionIndex;

  var _useNameGenerator = useNameGenerator(),
      createNameById = _useNameGenerator.createNameById;

  var removeElement = React__default.useCallback(function () {
    actions.removeFormElement(formElementId);
  }, [actions, formElementId]);
  var copyElement = React__default.useCallback(function () {
    actions.copyFormElement(formElementId, createNameById(formElementId), lastGroupPositionIndex + 1);
  }, [actions, createNameById, formElementId, lastGroupPositionIndex]);
  var swapElement = React__default.useCallback(function (groupID) {
    return function (indexA, indexB) {
      actions.swapFormElements(groupID, indexA, indexB);
    };
  }, [actions]);
  var transferElement = React__default.useCallback(function (groupId) {
    actions.transferElement(formElementId, groupId);
  }, [actions, formElementId]);
  return {
    removeElement: removeElement,
    copyElement: copyElement,
    swapElement: swapElement,
    transferElement: transferElement
  };
};

var useGrouping = function useGrouping() {
  var _useStore = useStore(function (s) {
    return s.grouping;
  }, function (a) {
    return a.groupingActions;
  }),
      groupings = _useStore[0],
      actions = _useStore[1];

  var addGroup = React__default.useCallback(function (name) {
    actions.addGroup({
      id: uuid(),
      name: name
    });
  }, [actions]);
  var setActiveGroup = React__default.useCallback(function (id) {
    actions.setActiveGroup(id);
  }, [actions]);
  var deleteGroup = React__default.useCallback(function () {
    actions.removeGroup(groupings.activeGroup);
  }, [actions, groupings.activeGroup]);
  var renameGroup = React__default.useCallback(function (name) {
    actions.renameGroup(groupings.activeGroup, name);
  }, [actions, groupings.activeGroup]);
  var swapGroups = React__default.useCallback(function () {
    actions.swapGroups.apply(actions, arguments);
  }, [actions]);
  var ActiveGroupName = React__default.useMemo(function () {
    var activeGroup = groupings.groups.find(function (g) {
      return g.id === groupings.activeGroup;
    });
    if (!activeGroup) return null;
    return activeGroup.name;
  }, [groupings.activeGroup, groupings.groups]);
  return {
    groupings: groupings,
    addGroup: addGroup,
    setActiveGroup: setActiveGroup,
    deleteGroup: deleteGroup,
    renameGroup: renameGroup,
    ActiveGroupName: ActiveGroupName,
    swapGroups: swapGroups
  };
};

var useValidationCreator = function useValidationCreator() {
  var _useFormElements = useFormElements(),
      allElements = _useFormElements.allElements;

  var ElementsValidations = React__default.useMemo(function () {
    return allElements.filter(function (el) {
      return JSON.stringify(el.validations) !== '{}';
    }).reduce(function (acc, el) {
      var _extends2;

      return _extends({}, acc, (_extends2 = {}, _extends2[el.name] = createValidationFieldSchema(getMappedValidations(el.validations), el.validationType), _extends2));
    }, {});
  }, [allElements]);
  var ValidationSchema = React__default.useMemo(function () {
    return yup.object().shape(ElementsValidations);
  }, [ElementsValidations]);
  return {
    data: {
      ValidationSchema: ValidationSchema
    }
  };
};
var getMappedValidations = function getMappedValidations(values) {
  var validations = [];
  Object.entries(values).forEach(function (_ref) {
    var key = _ref[0],
        value = _ref[1];

    if (!value) {
      return;
    }

    if (!key.includes('ErrorMessage')) {
      var validation;

      if (key === 'type') {
        validation = {
          type: values[key],
          params: [values[key + "ErrorMessage"]]
        };
      } else {
        validation = {
          type: key,
          params: [values[key + "ErrorMessage"]]
        };

        if (value && typeof value !== 'boolean') {
          validation.params.unshift(value);
        }
      }

      if (!(typeof value === 'boolean' && !value)) {
        validations.push(validation);
      }
    }
  });
  return validations;
};
var createValidationFieldSchema = function createValidationFieldSchema(validations, validationType) {
  var fieldSchema = new yup[validationType]().nullable();
  validations.forEach(function (validation) {
    var _fieldSchema;

    var params = validation.params,
        type = validation.type;

    if (!fieldSchema[type]) {
      return;
    }

    fieldSchema = (_fieldSchema = fieldSchema)[type].apply(_fieldSchema, params);
  });

  if (validationType === 'number') {
    fieldSchema = fieldSchema['transform'](emptyStringToNull);
  }

  return fieldSchema;
}; // helper for yup transform function

function emptyStringToNull(value, originalValue) {
  console.log({
    value: value,
    originalValue: originalValue
  });

  if (typeof originalValue === 'string' && originalValue === '') {
    return null;
  }

  return value;
}

var useRetractableString = function useRetractableString(original, retractable) {
  var _React$useState = React__default.useState(original),
      active = _React$useState[0],
      setActive = _React$useState[1];

  var retractString = function retractString() {
    setActive(retractable);
    setTimeout(function () {
      setActive(original);
    }, 3000);
  };

  return {
    text: active,
    retractString: retractString
  };
};

var useStyles$1 = /*#__PURE__*/core.makeStyles(function (theme) {
  return core.createStyles({
    previewContainer: {
      padding: theme.spacing(2),
      display: 'flex',
      height: '100%',
      flexDirection: 'column',
      alignItems: 'center',
      justifyContent: 'center'
    },
    attributes: {
      paddingTop: theme.spacing(3)
    }
  });
});
var AttributesEditor = function AttributesEditor(props) {
  var _useForm = reactHookForm.useForm({
    defaultValues: props.element.attributes
  }),
      register = _useForm.register,
      unregister = _useForm.unregister,
      control = _useForm.control,
      handleSubmit = _useForm.handleSubmit,
      watch = _useForm.watch,
      reset = _useForm.reset,
      setValue = _useForm.setValue;

  var watcher = watch();

  var _useNameGenerator = useNameGenerator(),
      createName = _useNameGenerator.createName;

  var _useFieldCreator = useFieldCreator(),
      getAttributeEditField = _useFieldCreator.getAttributeEditField;

  var _useFormElements = useFormElements(),
      setFormElementAttributes = _useFormElements.setFormElementAttributes,
      setFormElementAttribute = _useFormElements.setFormElementAttribute;

  var _useRetractableString = useRetractableString('Save Attributes', 'Saved!'),
      text = _useRetractableString.text,
      retractString = _useRetractableString.retractString;

  var classes = useStyles$1();

  var handleSave = function handleSave(data) {
    retractString();
    delete data[props.element.name];

    if (data.label !== props.element.attributes.label) {
      //there has been name change
      if (props.element.name.toLowerCase() === 'tree') {
        setFormElementAttribute(props.element.id, 'name', createName(data.label, 'tree'));
      } else {
        setFormElementAttribute(props.element.id, 'name', createName(data.label));
      }
    }

    setFormElementAttributes(props.element.id, data);
  };

  return React__default.createElement("form", {
    onSubmit: handleSubmit(handleSave)
  }, React__default.createElement(core.Paper, {
    className: classes.previewContainer,
    elevation: 5
  }, React__default.createElement(core.Typography, {
    color: 'secondary'
  }, "Preview"), React__default.createElement(props.element.render, _extends({}, props.element.attributes, {
    register: register,
    unregister: unregister,
    control: control,
    setValue: setValue,
    reset: reset
  }, watcher, {
    name: props.element.name
  }))), React__default.createElement(core.Grid, {
    item: true,
    container: true,
    direction: 'column',
    spacing: 4,
    className: classes.attributes
  }, props.element.attributeSchema.map(function (attr) {
    return React__default.createElement(core.Grid, {
      item: true,
      key: attr.name
    }, getAttributeEditField(attr, {
      register: register
    }, control));
  })), React__default.createElement(core.Box, {
    display: 'flex',
    alignItems: 'center',
    justifyContent: 'center',
    p: 3
  }, React__default.createElement(core.Button, {
    variant: 'contained',
    color: 'primary',
    type: 'submit'
  }, text)));
};

var DraggableField = function DraggableField(props) {
  var _useDrag = reactDnd.useDrag({
    item: _extends({}, props.formElement, {
      type: 'draggable-field'
    }),
    collect: function collect(monitor) {
      return {
        isDragging: monitor.isDragging()
      };
    }
  }),
      isDragging = _useDrag[0].isDragging,
      drag = _useDrag[1];

  var isDesktop = useMediaQuery('(min-width:600px)');
  return React__default.createElement(core.ListItem, {
    ref: drag,
    style: {
      opacity: isDragging ? 0.8 : 1,
      cursor: 'pointer'
    },
    onClick: props.onClick,
    divider: true
  }, React__default.createElement(core.ListItemAvatar, null, props.formElement.icon), isDesktop && React__default.createElement(core.ListItemText, {
    primary: props.formElement.label,
    secondary: props.formElement.description
  }));
};

var theme = /*#__PURE__*/core.createMuiTheme({
  palette: {
    primary: {
      main: purple[500]
    },
    secondary: {
      main: green[500]
    }
  }
});

var DraggableTag = /*#__PURE__*/React.forwardRef(function (_ref, ref) {
  var swap = _ref.swap,
      index = _ref.index,
      value = _ref.value,
      label = _ref.label,
      number = _ref.number,
      id = _ref.id,
      tabProps = _objectWithoutPropertiesLoose(_ref, ["swap", "index", "value", "label", "number", "id"]);

  return React.createElement(SortableWrapper, {
    id: id,
    index: index,
    swap: swap
  }, React.createElement(core.Tab, Object.assign({}, tabProps, {
    ref: ref,
    value: value,
    label: React.createElement(core.Box, {
      display: "flex",
      flexDirection: "row",
      alignItems: "center"
    }, React.createElement(core.Typography, null, label), React.createElement(core.Typography, {
      variant: "caption",
      color: "textSecondary",
      style: {
        marginLeft: theme.spacing(1)
      }
    }, number)),
    wrapped: true
  })));
});

var useStyles$2 = /*#__PURE__*/core.makeStyles(function (theme) {
  return {
    root: {
      zIndex: 10000,
      position: 'absolute',
      top: 0,
      bottom: 0,
      left: 0,
      right: 0,
      backgroundColor: 'rgba(0,0,0,0.1)',
      borderWidth: 3,
      borderStyle: 'dashed',
      borderRadius: theme.shape.borderRadius,
      borderColor: theme.palette.grey.A200,
      display: 'flex',
      justifyContent: 'center',
      alignItems: 'center'
    }
  };
});
var DropIndicator = function DropIndicator() {
  var classes = useStyles$2();
  return React__default.createElement("div", {
    className: classes.root
  }, React__default.createElement(core.Typography, {
    variant: 'h4',
    color: 'textSecondary'
  }, "Drop here"));
};

var GroupChooserDialog = function GroupChooserDialog(props) {
  var _useGrouping = useGrouping(),
      groupings = _useGrouping.groupings,
      ActiveGroupName = _useGrouping.ActiveGroupName;

  var _useState = React.useState(groupings.activeGroup || ""),
      group = _useState[0],
      setGroup = _useState[1];

  var handleChangeGroup = React__default.useCallback(function () {
    if (groupings.activeGroup === group) {
      props.onClose();
      return;
    }

    props.onSuccess(group);
    props.onClose();
  }, [group, groupings.activeGroup, props]);
  return React__default.createElement(CustomDialog, {
    opened: props.open,
    onSuccess: handleChangeGroup,
    onDeny: props.onClose,
    title: "Transfer " + ActiveGroupName + " to new group"
  }, React__default.createElement(core.FormControl, {
    style: {
      width: "100%"
    }
  }, React__default.createElement(core.InputLabel, {
    id: "group-select"
  }, "Group to transfer"), React__default.createElement(core.Select, {
    labelId: "group-select",
    id: "group-select",
    value: group,
    onChange: function onChange(e) {
      return setGroup(e.target.value);
    }
  }, groupings.groups.map(function (group) {
    return React__default.createElement(core.MenuItem, {
      value: group.id,
      key: group.id
    }, group.name);
  }))));
};

var useToolboxStyles = /*#__PURE__*/core.makeStyles(function (theme) {
  return core.createStyles({
    container: {
      display: "flex",
      flexDirection: "row",
      borderWidth: 1,
      borderStyle: "solid",
      borderColor: theme.palette.grey.A100,
      borderBottomLeftRadius: theme.shape.borderRadius,
      borderTopWidth: 0
    },
    actionItem: {
      padding: theme.spacing(0.2, 1, 0.2, 1),
      borderRightWidth: 1,
      borderRightStyle: "solid",
      borderRightColor: theme.palette.grey.A100,
      cursor: "pointer",
      "&:last-child": {
        borderRightWidth: 0
      },
      "&:hover": {
        backgroundColor: theme.palette.action.hover
      }
    }
  });
});
var FormElementToolbox = function FormElementToolbox(props) {
  var classes = useToolboxStyles();

  var _useDialog = useDialog(),
      open = _useDialog[0],
      handleOpen = _useDialog[1],
      handleClose = _useDialog[2];

  var _useFormElementAction = useFormElementActions(props.formElementId),
      removeElement = _useFormElementAction.removeElement,
      copyElement = _useFormElementAction.copyElement,
      transferElement = _useFormElementAction.transferElement;

  var commonActions = [{
    icon: React__default.createElement(icons.SwapHorizontalCircleOutlined, {
      color: "action"
    }),
    label: "Change Group",
    onClick: handleOpen
  }, {
    icon: React__default.createElement(icons.FileCopyOutlined, {
      color: "action"
    }),
    label: "Copy",
    onClick: copyElement
  }, {
    icon: React__default.createElement(icons.DeleteOutlined, {
      color: "error"
    }),
    label: "Delete",
    onClick: removeElement
  }];
  var actions = props.actions ? [].concat(props.actions, commonActions) : [].concat(commonActions);
  return React__default.createElement("div", {
    className: classes.container
  }, actions.map(function (action, i) {
    return React__default.createElement(core.Tooltip, {
      title: action.label,
      key: i
    }, React__default.createElement("div", {
      className: classes.actionItem,
      onClick: action.onClick,
      "data-cy": "action-" + action.label
    }, action.icon));
  }), React__default.createElement(GroupChooserDialog, {
    open: open,
    onSuccess: transferElement,
    onClose: handleClose
  }));
};

var useStyles$3 = /*#__PURE__*/styles.makeStyles(function (theme) {
  return core.createStyles({
    spacing: {
      padding: theme.spacing(1)
    },
    dragArea: {
      cursor: 'move',
      padding: theme.spacing(0.2, 1, 0.2, 1)
    },
    center: {
      display: 'flex',
      justifyContent: 'center',
      alignItems: 'center'
    },
    container: {
      width: '100%',
      borderWidth: 1,
      borderStyle: 'solid',
      borderColor: theme.palette.grey.A100
    },
    elementContainer: {
      padding: theme.spacing(0, 2, 1, 2)
    }
  });
});
var FormBuildElement = function FormBuildElement(props) {
  var _useFormElementAction = useFormElementActions(props.element.id),
      swapElement = _useFormElementAction.swapElement;

  var _useDialog = useDialog(false),
      open = _useDialog[0],
      handleOpen = _useDialog[1],
      handleClose = _useDialog[2];

  var classes = useStyles$3();
  return React__default.createElement("div", {
    style: {
      width: '100%',
      marginBottom: 10
    }
  }, React__default.createElement(SortableWrapper, {
    id: props.element.id,
    index: props.index,
    swap: swapElement(props.element.groupID)
  }, React__default.createElement(core.Paper, {
    className: classes.container,
    style: {
      width: '100%'
    },
    elevation: 0
  }, React__default.createElement(core.Grid, {
    container: true,
    direction: 'column'
  }, React__default.createElement(core.Grid, {
    item: true,
    container: true,
    direction: 'row',
    justify: 'space-between'
  }, React__default.createElement(core.Grid, {
    item: true
  }, React__default.createElement("div", {
    className: classes.dragArea
  }, React__default.createElement(icons.DragHandleOutlined, {
    color: 'action',
    className: classes.center
  }))), React__default.createElement(core.Grid, {
    item: true,
    className: classes.spacing
  }, React__default.createElement(core.Typography, {
    color: 'secondary',
    variant: 'overline'
  }, props.element.label)), React__default.createElement(core.Grid, {
    item: true
  }, React__default.createElement(FormElementToolbox, {
    formElementId: props.element.id,
    actions: !props.element.isReadOnly ? [{
      icon: React__default.createElement(icons.EditOutlined, {
        color: 'action'
      }),
      label: 'Edit',
      onClick: handleOpen
    }] : undefined
  }))), React__default.createElement(core.Grid, {
    item: true,
    container: true,
    direction: 'row',
    alignItems: 'center',
    spacing: 1
  }, React__default.createElement(core.Grid, {
    item: true,
    xs: 12
  }, React__default.createElement("div", {
    className: classes.elementContainer
  }, React__default.createElement(props.element.render, _extends({}, props.element.attributes, props.formContextRegistration, {
    name: props.element.name
  })))))))), React__default.createElement(FormElementEditWindow, {
    open: open,
    onClose: handleClose,
    element: props.element
  }));
};

var useStyles$4 = /*#__PURE__*/core.makeStyles(function (theme) {
  return core.createStyles({
    container: {
      "&:hover $dragContainer": {
        visibility: "visible"
      }
    },
    dragContainer: {
      visibility: "hidden",
      cursor: "move",
      position: "relative",
      display: "flex",
      alignItems: "center",
      height: "100%",
      left: theme.spacing(0.5)
    },
    wrapper: {
      borderWidth: 1,
      borderStyle: "solid",
      borderColor: theme.palette.grey.A100,
      borderRadius: theme.shape.borderRadius,
      padding: theme.spacing(2)
    },
    actionArea: {
      display: "flex",
      justifyContent: "center",
      alignItems: "center"
    },
    actionDivider: {
      paddingLeft: theme.spacing(1),
      paddingRight: theme.spacing(1)
    }
  });
});
var OptionsBuilder = function OptionsBuilder(props) {
  var classes = useStyles$4();

  var _useFieldArray = reactHookForm.useFieldArray({
    name: props.name,
    control: props.control
  }),
      fields = _useFieldArray.fields,
      append = _useFieldArray.append,
      remove = _useFieldArray.remove,
      swap = _useFieldArray.swap;

  return React.createElement(React.Fragment, null, React.createElement(core.Grid, {
    container: true,
    spacing: 1,
    direction: "column",
    className: classes.wrapper
  }, React.createElement(core.Typography, {
    color: "textSecondary",
    variant: "subtitle2"
  }, "options"), fields.map(function (item, index) {
    return React.createElement(SortableWrapper, {
      id: item.id,
      index: index,
      swap: swap,
      dndIndicator: "optionItemSortable",
      key: item.id
    }, React.createElement(core.Grid, {
      item: true,
      container: true,
      direction: "row",
      alignItems: "center",
      justify: "center",
      spacing: 2,
      className: classes.container
    }, React.createElement("div", {
      className: classes.dragContainer
    }, React.createElement(icons.DragIndicatorOutlined, {
      color: "action"
    })), React.createElement(core.Grid, {
      item: true,
      xs: 6
    }, React.createElement(core.Input, {
      fullWidth: true,
      placeholder: "Option " + (index + 1),
      autoFocus: true,
      onFocus: function onFocus(event) {
        event.target.select();
      },
      inputRef: props.register(),
      defaultValue: item.name,
      name: props.name + "[" + index + "].name"
    })), React.createElement(core.Grid, {
      item: true,
      xs: 2
    }, React.createElement(core.Tooltip, {
      title: "Remove option",
      "aria-label": "remove-option",
      enterDelay: 500
    }, React.createElement(core.IconButton, {
      "aria-label": "delete",
      onClick: function onClick() {
        return remove(index);
      }
    }, React.createElement(icons.Clear, null))))));
  }), React.createElement(core.Box, {
    p: 2,
    className: classes.actionArea
  }, React.createElement(core.Button, {
    color: "primary",
    variant: "outlined",
    onClick: function onClick() {
      return append({
        name: ""
      });
    }
  }, "Add Option"), React.createElement("div", {
    className: classes.actionDivider
  }, React.createElement(core.Typography, {
    color: "textSecondary"
  }, "or")), React.createElement(core.Button, {
    variant: "text",
    size: "small",
    onClick: function onClick() {
      return append({
        name: "other",
        label: "Other"
      });
    }
  }, "Add 'Other' Option"))));
};

var validatorsMapper = {
  required: Required,
  min: Min,
  max: Max,
  length: Length,
  matches: Regex,
  email: Email,
  url: Url,
  moreThan: MoreThan,
  lessThan: LessThan,
  positive: Positive,
  negative: Negative,
  integer: Integer
};

var useStyles$5 = /*#__PURE__*/core.makeStyles(function (theme) {
  return core.createStyles({
    validator: {
      marginBottom: theme.spacing(2)
    }
  });
});
var ValidationsEditor = function ValidationsEditor(props) {
  var _props$element$valida;

  var _useForm = reactHookForm.useForm({
    defaultValues: props.element.validations
  }),
      register = _useForm.register,
      handleSubmit = _useForm.handleSubmit,
      watch = _useForm.watch;

  useValidationCreator();
  var watcher = watch();

  var _useFormElements = useFormElements(),
      handlers = _useFormElements.handlers;

  var classes = useStyles$5();

  var handleSave = function handleSave(data) {
    handlers.setFormElementValidations(props.element.id, data);
  };

  return React__default.createElement("form", {
    onSubmit: handleSubmit(handleSave)
  }, React__default.createElement(core.Grid, {
    item: true,
    container: true,
    direction: 'column',
    spacing: 2
  }, (_props$element$valida = props.element.validators) == null ? void 0 : _props$element$valida.map(function (validator) {
    return React__default.createElement(core.Grid, {
      item: true,
      key: validator
    }, React__default.createElement(validatorsMapper[validator], {
      register: register,
      disabledParam: !watcher[validator],
      name: validator,
      className: classes.validator
    }));
  })), React__default.createElement(core.Box, {
    display: 'flex',
    alignItems: 'center',
    justifyContent: 'center',
    p: 3
  }, React__default.createElement(core.Button, {
    variant: 'contained',
    color: 'primary',
    type: 'submit'
  }, "Save Validations")));
};

var useStyles$6 = /*#__PURE__*/core.makeStyles(function (theme) {
  return core.createStyles({
    root: {
      margin: 0,
      padding: theme.spacing(2)
    },
    closeButton: {
      position: 'absolute',
      right: theme.spacing(1),
      top: theme.spacing(1)
    }
  });
});
var FormElementEditWindow = function FormElementEditWindow(props) {
  var classes = useStyles$6();
  return React__default.createElement(React__default.Fragment, null, React__default.createElement(core.Dialog, {
    open: props.open,
    onClose: props.onClose,
    fullWidth: true,
    maxWidth: 'sm'
  }, React__default.createElement(core.DialogTitle, {
    disableTypography: true,
    className: classes.root
  }, React__default.createElement(core.Typography, {
    variant: "h6"
  }, props.element.label, " Editor"), React__default.createElement(core.IconButton, {
    "aria-label": "close",
    onClick: props.onClose,
    className: classes.closeButton,
    "data-cy": 'button-close-modal'
  }, React__default.createElement(CloseIcon, null))), React__default.createElement(core.DialogContent, null, React__default.createElement(core.Box, null, React__default.createElement(core.Box, null, React__default.createElement(core.Box, {
    marginBottom: 1
  }, React__default.createElement(core.Typography, {
    variant: 'h6',
    color: 'primary',
    gutterBottom: true
  }, "Attributes"), React__default.createElement(core.Divider, null)), React__default.createElement(AttributesEditor, {
    element: props.element
  })), props.element.validators && React__default.createElement(core.Box, null, React__default.createElement(core.Box, {
    marginBottom: 2
  }, React__default.createElement(core.Typography, {
    variant: 'h6',
    color: 'primary',
    gutterBottom: true
  }, "Validations"), React__default.createElement(core.Divider, null)), React__default.createElement(ValidationsEditor, {
    element: props.element
  }))))));
};

var UploadJSONButton = function UploadJSONButton(_ref) {
  var onJson = _ref.onJson,
      imported = _ref.imported;

  var _React$useState = React__default.useState(false),
      isLoading = _React$useState[0],
      setIsLoading = _React$useState[1];

  var uploadInputRef = React__default.useRef(null);

  var handleCapture = function handleCapture(_ref2) {
    var target = _ref2.target;

    if (!target || !target.files) {
      return;
    }

    setIsLoading(true);
    var fileReader = new FileReader();
    fileReader.readAsText(target.files[0]);

    fileReader.onload = function (e) {
      var _e$target, _e$target$result;

      var jsonString = (_e$target = e.target) == null ? void 0 : (_e$target$result = _e$target.result) == null ? void 0 : _e$target$result.toString();
      var json = JSON.parse(jsonString != null ? jsonString : '');
      setIsLoading(false);
      onJson == null ? void 0 : onJson(json);
    };
  };

  return React__default.createElement(React__default.Fragment, null, React__default.createElement("input", {
    ref: uploadInputRef,
    type: "file",
    accept: "application/JSON",
    style: {
      display: 'none'
    },
    onChange: handleCapture
  }), React__default.createElement(core.Button, {
    onClick: function onClick() {
      var _uploadInputRef$curre;

      return uploadInputRef == null ? void 0 : (_uploadInputRef$curre = uploadInputRef.current) == null ? void 0 : _uploadInputRef$curre.click();
    },
    variant: "text",
    color: imported ? 'default' : 'secondary'
  }, isLoading && React__default.createElement(core.CircularProgress, null), imported ? 'Imported' : 'Import'));
};

var RfaSchemaTransferer = function RfaSchemaTransferer(_ref3) {
  var open = _ref3.open,
      onClose = _ref3.onClose;

  var _useDataExporter = useDataExporter(),
      handlers = _useDataExporter.handlers,
      state = _useDataExporter.state;

  var classes = useStyles$7();
  return React__default.createElement(core.Dialog, {
    open: open,
    onClose: onClose
  }, React__default.createElement(core.DialogTitle, {
    className: classes.root,
    disableTypography: true
  }, React__default.createElement(core.Typography, {
    variant: "h6"
  }, 'RFA Transferer'), React__default.createElement(core.IconButton, {
    "aria-label": "close",
    className: classes.closeButton,
    onClick: onClose
  }, React__default.createElement(icons.Close, null))), React__default.createElement(core.DialogContent, null, React__default.createElement(core.Typography, {
    color: 'textSecondary'
  }, "Export form data as JSON file and/or import it to react form architect - renderer"), React__default.createElement(core.Box, {
    display: 'flex',
    flexDirection: 'row',
    alignItems: 'center',
    justifyContent: 'space-around',
    className: classes.root
  }, React__default.createElement(core.Button, {
    onClick: handlers.saveAsJSON,
    variant: "text",
    color: state.exported ? 'default' : 'secondary'
  }, state.exported ? 'Exported' : 'Export as JSON'), React__default.createElement(core.Typography, {
    className: classes.divider
  }, " or "), React__default.createElement(UploadJSONButton, {
    onJson: handlers.importData,
    imported: state.imported
  }), React__default.createElement(core.Typography, {
    className: classes.divider
  }, " or "), React__default.createElement(core.Button, {
    onClick: handlers.copyToClipboard,
    variant: "text",
    color: state.copied ? 'default' : 'secondary'
  }, state.copied ? 'Copied' : 'Copy to Clipboard'))));
};
var useStyles$7 = /*#__PURE__*/styles.makeStyles(function (theme) {
  return core.createStyles({
    root: {
      margin: 0,
      padding: theme.spacing(2)
    },
    closeButton: {
      position: 'absolute',
      right: theme.spacing(1),
      top: theme.spacing(1),
      color: theme.palette.grey[500]
    },
    divider: {
      marginLeft: theme.spacing(1),
      marginRight: theme.spacing(1)
    }
  });
});

var FieldBox = function FieldBox(_ref) {
  var formElements = _ref.formElements;

  var _useFormElements = useFormElements(),
      createFormElement = _useFormElements.createFormElement;

  var handleClick = React__default.useCallback(function (formElement) {
    return createFormElement(formElement);
  }, [createFormElement]);
  return React__default.createElement(core.Paper, {
    variant: 'outlined'
  }, React__default.createElement(core.List, {
    disablePadding: true,
    dense: true
  }, formElements.map(function (formElement, i) {
    return React__default.createElement(DraggableField, {
      key: i,
      formElement: formElement,
      onClick: function onClick() {
        return handleClick(formElement);
      }
    });
  })));
};

var FormElementsContainer = function FormElementsContainer() {
  var _useForm = reactHookForm.useForm(),
      register = _useForm.register,
      control = _useForm.control,
      setValue = _useForm.setValue,
      unregister = _useForm.unregister;

  var _useFormElements = useFormElements(),
      elements = _useFormElements.elements;

  return React__default.createElement(core.Box, {
    p: 1,
    width: '100%',
    height: '100%'
  }, React__default.createElement("form", null, elements.map(function (element, index) {
    return React__default.createElement(FormBuildElement, {
      key: element.id,
      element: element,
      index: index,
      formContextRegistration: {
        register: register,
        control: control,
        setValue: setValue,
        unregister: unregister
      }
    });
  })));
};

function TabPanel(props) {
  var children = props.children,
      value = props.value,
      index = props.index,
      other = _objectWithoutPropertiesLoose(props, ["children", "value", "index"]);

  return React.createElement("div", Object.assign({
    role: "tabpanel",
    hidden: value !== index,
    id: "simple-tabpanel-" + index,
    "aria-labelledby": "simple-tab-" + index
  }, other), value === index && children);
}

var useStyles$8 = /*#__PURE__*/core.makeStyles({
  root: {
    position: 'relative',
    display: 'flex',
    flex: '1 1 0',
    alignItems: 'center',
    justifyContent: 'flex-start',
    flexDirection: 'column'
  },
  contentWrapper: {
    padding: 10
  },
  dropArea: {
    width: '100%',
    justifyContent: 'center'
  },
  buttonGroup: {
    justifyContent: 'flex-end',
    paddingBottom: 5
  },
  dataTransferButtonGroup: {
    justifyContent: 'flex-start',
    paddingBottom: 5
  }
});
var DropArea = function DropArea() {
  var _React$useState = React.useState(''),
      customizedGroupName = _React$useState[0],
      setCustomizedGroupName = _React$useState[1];

  var _React$useState2 = React.useState(false),
      dropIndicationActive = _React$useState2[0],
      setDropIndicationActive = _React$useState2[1];

  var classes = useStyles$8();

  var _useGrouping = useGrouping(),
      groupings = _useGrouping.groupings,
      addGroup = _useGrouping.addGroup,
      setActiveGroup = _useGrouping.setActiveGroup,
      renameGroup = _useGrouping.renameGroup,
      deleteGroup = _useGrouping.deleteGroup,
      ActiveGroupName = _useGrouping.ActiveGroupName,
      swapGroups = _useGrouping.swapGroups;

  var _useFormElements = useFormElements(),
      createFormElement = _useFormElements.createFormElement,
      allElements = _useFormElements.allElements;

  var _useDialog = useDialog(),
      openDelete = _useDialog[0],
      handleOpenDelete = _useDialog[1],
      handleCloseDelete = _useDialog[2];

  var _useDialog2 = useDialog(),
      openRename = _useDialog2[0],
      handleOpenRename = _useDialog2[1],
      handleCloseRename = _useDialog2[2];

  var _useDialog3 = useDialog(),
      openTransfer = _useDialog3[0],
      handleOpenTransfer = _useDialog3[1],
      handleCloseTransfer = _useDialog3[2];

  var _useDrop = reactDnd.useDrop({
    accept: 'draggable-field',
    drop: function drop(dropItem) {
      createFormElement(dropItem);
    },
    collect: function collect(monitor) {
      setDropIndicationActive(monitor.canDrop());
    }
  }),
      drop = _useDrop[1];

  var handleChange = React.useCallback(function (_, newValue) {
    setActiveGroup(newValue);
  }, [setActiveGroup]);
  var handleAddGroup = React.useCallback(function () {
    addGroup("Group " + groupings.groups.length);
  }, [addGroup, groupings.groups.length]);
  var handleGroupRename = React.useCallback(function () {
    renameGroup(customizedGroupName);
    setCustomizedGroupName('');
    handleCloseRename();
  }, [customizedGroupName, handleCloseRename, renameGroup]);
  var handleGroupRenameOpen = React.useCallback(function () {
    setCustomizedGroupName(ActiveGroupName || '');
    handleOpenRename();
  }, [ActiveGroupName, handleOpenRename]);
  var handleGroupDelete = React.useCallback(function () {
    deleteGroup();
    handleCloseDelete();
  }, [deleteGroup, handleCloseDelete]);
  var getGroupNumberOfGroupElements = React.useCallback(function (groupId) {
    return allElements.filter(function (el) {
      return el.groupID === groupId;
    }).length;
  }, [allElements]);
  return React.createElement(core.Box, {
    display: 'flex',
    flexDirection: 'column',
    style: {
      height: '100%',
      width: '100%'
    }
  }, React.createElement(core.Box, {
    width: '100%',
    display: 'flex',
    flexDirection: 'row',
    justifyContent: 'space-between',
    paddingBottom: 1
  }, React.createElement(core.Button, {
    variant: 'outlined',
    onClick: handleOpenTransfer
  }, "Data Transfer"), React.createElement(core.ButtonGroup, {
    color: "primary",
    size: 'small',
    "aria-label": "outlined primary button group"
  }, React.createElement(core.Button, {
    onClick: handleAddGroup
  }, "Add Group"), React.createElement(core.Button, {
    onClick: handleGroupRenameOpen
  }, "Rename Active Group"), ",", groupings.groups.length > 1 && React.createElement(core.Button, {
    onClick: handleOpenDelete
  }, "Delete Group"))), React.createElement(core.Paper, {
    ref: drop,
    className: classes.root,
    variant: 'outlined'
  }, groupings.groups.length > 0 && React.createElement(core.AppBar, {
    position: "static",
    color: "default",
    elevation: 0
  }, React.createElement(core.Tabs, {
    value: groupings.activeGroup,
    onChange: handleChange,
    variant: "scrollable",
    scrollButtons: 'auto',
    "aria-label": "simple tabs example"
  }, groupings.groups.map(function (group, i) {
    return React.createElement(DraggableTag, {
      key: group.id,
      value: group.id,
      label: group.name,
      number: getGroupNumberOfGroupElements(group.id),
      id: group.id,
      index: i,
      swap: swapGroups
    });
  }))), dropIndicationActive && React.createElement(DropIndicator, null), groupings.groups.map(function (group) {
    return React.createElement(TabPanel, {
      key: group.id,
      value: groupings.activeGroup,
      index: group.id,
      className: classes.dropArea
    }, React.createElement(FormElementsContainer, null));
  })), React.createElement(ConfirmationDialog, {
    opened: openDelete,
    title: "Are you sure to delete " + ActiveGroupName + "?",
    text: 'You will lose its form progress',
    onSuccess: handleGroupDelete,
    onDeny: handleCloseDelete
  }), React.createElement(CustomDialog, {
    opened: openRename,
    title: "Enter new name for " + ActiveGroupName,
    onSuccess: handleGroupRename,
    onDeny: handleCloseRename
  }, React.createElement(core.TextField, {
    autoFocus: true,
    value: customizedGroupName,
    onChange: function onChange(e) {
      return setCustomizedGroupName(e.target.value);
    },
    label: "New name",
    margin: "dense",
    id: "name"
  })), React.createElement(RfaSchemaTransferer, {
    open: openTransfer,
    onClose: handleCloseTransfer
  }));
};

var FormArchitectSaveButton = function FormArchitectSaveButton(_ref) {
  var onSave = _ref.onSave;

  var _useStore = useStore(function (s) {
    return s;
  }),
      store = _useStore[0];

  var handleSave = React__default.useCallback(function () {
    onSave(store);
  }, [store]);
  return React__default.createElement(core.Button, {
    color: 'primary',
    variant: 'contained',
    onClick: handleSave,
    disabled: !store.elements.length
  }, "Save Form");
};

var useIntersectionObserver = function useIntersectionObserver() {
  var _React$useState = React__default.useState(false),
      isSticky = _React$useState[0],
      setIsSticky = _React$useState[1];

  var ref = React__default.createRef();
  React__default.useEffect(function () {
    if (!ref.current) {
      return;
    }

    var cachedRef = ref.current,
        observer = new IntersectionObserver(function (_ref) {
      var e = _ref[0];
      return setIsSticky(e.intersectionRatio < 1);
    }, {
      threshold: [1],
      rootMargin: "-1px 0px 150px 0px"
    });
    observer.observe(cachedRef); // unmount

    return function () {
      observer.unobserve(cachedRef);
    };
  }, [ref]);
  return {
    isSticky: isSticky,
    ref: ref
  };
};

var TreeNodeTitleInput = function TreeNodeTitleInput(_ref) {
  var rest = _extends({}, _ref);

  return React__default.createElement(core.TextField, Object.assign({}, rest, {
    variant: "outlined",
    InputLabelProps: {
      shrink: true
    }
  }));
};

var TreeNodeDataInput = function TreeNodeDataInput(_ref) {
  var data = _ref.data,
      formContext = _ref.formContext,
      allowUserDefinedData = _ref.allowUserDefinedData;
  var classes = useStyles$9();

  var _React$useState = React__default.useState(),
      customDataObj = _React$useState[0],
      setCustomDataObj = _React$useState[1];

  var addCustomData = React__default.useCallback(function () {
    setCustomDataObj(function (prevState) {
      var _extends2;

      return _extends({}, prevState, (_extends2 = {}, _extends2[uuid()] = '', _extends2));
    });
  }, []);
  var removeCustomData = React__default.useCallback(function (id) {
    return function () {
      setCustomDataObj(function (prevState) {
        if (!prevState) {
          return prevState;
        }

        var out = _extends({}, prevState);

        delete out[id];
        return out;
      });
    };
  }, []);
  return React__default.createElement(core.Box, {
    paddingTop: 1
  }, data == null ? void 0 : data.map(function (payloadEntry, index) {
    var _payloadEntry$value, _payloadEntry$placeho;

    return React__default.createElement(core.Box, {
      className: classes.payloadEntry,
      key: index
    }, React__default.createElement(core.Typography, {
      variant: 'caption',
      className: classes.payloadCaption
    }, payloadEntry.name + ":"), payloadEntry.value ? React__default.createElement(core.Typography, null, payloadEntry.value) : React__default.createElement(core.Input, {
      inputRef: formContext.register,
      name: formContext.name + ".data." + payloadEntry.name,
      defaultValue: (_payloadEntry$value = payloadEntry.value) != null ? _payloadEntry$value : '',
      placeholder: (_payloadEntry$placeho = payloadEntry.placeholder) != null ? _payloadEntry$placeho : '',
      inputProps: {
        'aria-label': 'description'
      }
    }));
  }), allowUserDefinedData && customDataObj && Object.entries(customDataObj).map(function (_ref2, index) {
    var id = _ref2[0];
    return React__default.createElement(core.Box, {
      className: classes.payloadEntry,
      key: id
    }, React__default.createElement(core.Input, {
      className: classes.payloadCaption,
      inputRef: formContext.register,
      name: formContext.name + ".userData[" + index + "].name",
      inputProps: {
        'aria-label': 'name'
      },
      endAdornment: ':',
      placeholder: 'name',
      autoFocus: true
    }), React__default.createElement(core.Input, {
      inputRef: formContext.register,
      name: formContext.name + ".userData[" + index + "].value",
      placeholder: 'value',
      inputProps: {
        'aria-label': 'description'
      }
    }), React__default.createElement(core.IconButton, {
      "aria-label": "delete",
      onClick: removeCustomData(id),
      size: 'small'
    }, React__default.createElement(icons.Clear, {
      color: 'error',
      fontSize: 'small'
    })));
  }), allowUserDefinedData && React__default.createElement(core.Box, {
    className: classes.actionArea
  }, React__default.createElement(core.Button, {
    variant: 'outlined',
    color: 'secondary',
    onClick: addCustomData,
    startIcon: React__default.createElement(icons.Add, null),
    size: 'small'
  }, "data entry")));
};
var useStyles$9 = /*#__PURE__*/core.makeStyles(function (theme) {
  return core.createStyles({
    payloadCaption: {
      marginRight: theme.spacing(1),
      width: theme.spacing(13)
    },
    payloadEntry: {
      display: 'flex',
      flexDirection: 'row',
      alignItems: 'center',
      borderBottomWidth: 1,
      borderBottomStyle: 'solid',
      borderBottomColor: theme.palette.grey['200']
    },
    actionArea: {
      marginTop: theme.spacing(1),
      display: 'flex',
      justifyContent: 'center'
    }
  });
});

var NodeTitleAnchor = function NodeTitleAnchor(_ref) {
  var visible = _ref.visible,
      title = _ref.title,
      _onClick = _ref.onClick;
  var classes = useStyles$a();
  return React__default.createElement("div", {
    className: classes.root,
    onClick: function onClick(e) {
      e.preventDefault();
      _onClick == null ? void 0 : _onClick();
    }
  }, React__default.createElement(core.Typography, {
    color: 'textSecondary',
    className: classes.nodeTitle,
    variant: 'caption',
    style: {
      visibility: visible ? 'visible' : 'hidden'
    }
  }, title));
};
var useStyles$a = /*#__PURE__*/core.makeStyles(function (theme) {
  return core.createStyles({
    root: {
      position: 'sticky',
      top: 0,
      zIndex: 15,
      cursor: 'pointer'
    },
    nodeTitle: {
      border: "1px " + theme.palette.primary.light + " solid",
      borderRadius: theme.shape.borderRadius,
      backgroundColor: theme.palette.background["default"],
      position: 'absolute',
      height: 'max-content',
      writingMode: 'vertical-rl',
      padding: theme.spacing(0.5, 0.25, 0.5, 0.25),
      textShadow: "0px 0px 1px " + theme.palette.primary.light,
      '&:hover': {
        border: "1px " + theme.palette.primary.main + " solid",
        textShadow: "0px 0px 1px " + theme.palette.primary.main
      }
    }
  });
});

var iconMapper = {
  remove: icons.Remove,
  add: icons.Add,
  expandMore: icons.ExpandMore,
  expandLess: icons.ExpandLess
};
var TreeNodeCornerButton = function TreeNodeCornerButton(_ref) {
  var corner = _ref.corner,
      icon = _ref.icon,
      _ref$color = _ref.color,
      color = _ref$color === void 0 ? 'inherit' : _ref$color,
      onClick = _ref.onClick;
  var classes = useStyles$b();
  var handleClick = React__default.useCallback(function (e) {
    e.preventDefault();
    onClick();
  }, [onClick]);
  return React__default.createElement("div", {
    className: classes[corner]
  }, React__default.createElement("div", {
    className: classes.button,
    style: {
      borderColor: theme.palette.grey['300']
    },
    onClick: handleClick
  }, React__default.createElement(iconMapper[icon], {
    color: color
  })));
};
var size = 2.5;
var useStyles$b = /*#__PURE__*/core.makeStyles(function (theme) {
  return core.createStyles({
    button: {
      height: theme.spacing(size),
      width: theme.spacing(size),
      cursor: 'pointer',
      borderRadius: theme.shape.borderRadius,
      backgroundColor: theme.palette.background["default"],
      borderWidth: 1,
      borderStyle: 'solid',
      display: 'flex',
      alignItems: 'center',
      justifyContent: 'center',
      '&:hover': {
        backgroundColor: theme.palette.grey.A100
      }
    },
    topRight: {
      position: 'absolute',
      top: -theme.spacing(size) / 2,
      right: -theme.spacing(size) / 2,
      zIndex: 10
    },
    bottomLeft: {
      position: 'absolute',
      bottom: -theme.spacing(size) / 2,
      left: -theme.spacing(size) / 2,
      zIndex: 10
    },
    bottomRight: {
      position: 'absolute',
      bottom: -theme.spacing(size) / 2,
      right: -theme.spacing(size) / 2,
      zIndex: 10
    }
  });
});

var useClientRect = function useClientRect() {
  var _React$useState = React__default.useState(),
      rect = _React$useState[0],
      setRect = _React$useState[1];

  var _React$useState2 = React__default.useState(),
      nodeElement = _React$useState2[0],
      setNodeElement = _React$useState2[1];

  var ref = React__default.useCallback(function (node) {
    if (node !== null) {
      setRect(node.getBoundingClientRect());
      setNodeElement(node);
    }
  }, []);
  return [rect, ref, nodeElement];
};

var TreeNodeInput = function TreeNodeInput(_ref) {
  var _nodeRect$height, _nodeRect$height2;

  var id = _ref.id,
      formContext = _ref.formContext,
      data = _ref.data,
      descendents = _ref.descendents,
      allowUserToDefineData = _ref.allowUserToDefineData,
      _ref$isEndingNode = _ref.isEndingNode,
      isEndingNode = _ref$isEndingNode === void 0 ? false : _ref$isEndingNode,
      _ref$level = _ref.level,
      level = _ref$level === void 0 ? 1 : _ref$level,
      onDelete = _ref.onDelete,
      nodeLabel = _ref.nodeLabel,
      textInputProps = _objectWithoutPropertiesLoose(_ref, ["id", "formContext", "data", "descendents", "allowUserToDefineData", "isEndingNode", "level", "onDelete", "nodeLabel", "required"]);

  var classes = useStyles$c();

  var _React$useState = React__default.useState(true),
      isOpen = _React$useState[0],
      setIsOpen = _React$useState[1];

  var _React$useState2 = React__default.useState(),
      childNodes = _React$useState2[0],
      setChildNodes = _React$useState2[1];

  var hasDescendents = React__default.useMemo(function () {
    return !!((descendents == null ? void 0 : descendents.length) || (childNodes == null ? void 0 : childNodes.length));
  }, [childNodes, descendents]);
  var toggleDescendents = React__default.useCallback(function () {
    if (!hasDescendents) {
      return;
    }

    setIsOpen(function (prevState) {
      return !prevState;
    });
  }, [hasDescendents]);

  var _useIntersectionObser = useIntersectionObserver(),
      ref = _useIntersectionObser.ref,
      isSticky = _useIntersectionObser.isSticky;

  var nameValue = reactHookForm.useWatch({
    name: formContext.name + ".name",
    control: formContext.control,
    defaultValue: "Node level " + level
  });

  var _useClientRect = useClientRect(),
      nodeRect = _useClientRect[0],
      nodeRef = _useClientRect[1],
      nodeElement = _useClientRect[2];

  var scrollToNode = React__default.useCallback(function () {
    if (!nodeElement) {
      return;
    }

    nodeElement.scrollIntoView({
      behavior: 'smooth'
    });
  }, [nodeElement]);
  var addChildNode = React__default.useCallback(function () {
    if (!isOpen) {
      setIsOpen(true);
    }

    setChildNodes(function (prevState) {
      return [].concat(prevState != null ? prevState : [], [{
        id: uuid()
      }]);
    });
  }, [isOpen]);
  var removeChildNode = React__default.useCallback(function () {
    onDelete == null ? void 0 : onDelete(id);
  }, [id, onDelete]);
  var onChildRemove = React__default.useCallback(function (id) {
    setChildNodes(function (prevState) {
      return prevState == null ? void 0 : prevState.filter(function (childNode) {
        return childNode.id !== id;
      });
    });
  }, []);
  return React__default.createElement(React__default.Fragment, null, React__default.createElement("div", {
    className: classes.root,
    style: level === 1 ? {
      paddingLeft: 0
    } : {}
  }, level !== 1 && React__default.createElement("div", {
    className: classes.edgeWrapper
  }, React__default.createElement("div", {
    className: classes.edgeVer,
    style: {
      height: !isEndingNode ? '100%' : ((_nodeRect$height = nodeRect == null ? void 0 : nodeRect.height) != null ? _nodeRect$height : 2) / 2
    }
  }), React__default.createElement("div", {
    className: classes.edgeHor,
    style: {
      top: ((_nodeRect$height2 = nodeRect == null ? void 0 : nodeRect.height) != null ? _nodeRect$height2 : 2) / 2
    }
  })), React__default.createElement(NodeTitleAnchor, {
    visible: !!(isSticky && hasDescendents && isOpen),
    title: nameValue,
    onClick: scrollToNode
  }), React__default.createElement(core.Paper, {
    variant: 'outlined',
    className: classes.node,
    innerRef: nodeRef
  }, React__default.createElement(TreeNodeTitleInput, Object.assign({}, textInputProps, {
    label: nodeLabel,
    inputRef: formContext.register,
    name: formContext.name + ".name",
    style: {
      display: 'flex',
      flex: 1
    }
  })), React__default.createElement(TreeNodeCornerButton, {
    onClick: addChildNode,
    icon: 'add',
    corner: 'bottomRight',
    color: 'action'
  }), onDelete && React__default.createElement(TreeNodeCornerButton, {
    onClick: removeChildNode,
    icon: 'remove',
    corner: 'topRight',
    color: 'error'
  }), hasDescendents && React__default.createElement(TreeNodeCornerButton, {
    onClick: toggleDescendents,
    icon: isOpen ? 'expandLess' : 'expandMore',
    corner: 'bottomLeft',
    color: 'action'
  }), React__default.createElement("div", {
    ref: ref
  }), React__default.createElement(TreeNodeDataInput, {
    data: data,
    allowUserDefinedData: allowUserToDefineData,
    formContext: formContext
  })), React__default.createElement(core.Collapse, {
    "in": isOpen
  }, childNodes == null ? void 0 : childNodes.map(function (descendant, index) {
    return React__default.createElement(TreeNodeInput, Object.assign({
      key: descendant.id,
      id: descendant.id,
      formContext: {
        register: formContext.register,
        name: formContext.name + ".descendents[" + index + "]",
        control: formContext.control
      },
      data: data,
      descendents: descendant.descendents,
      level: level + 1,
      isEndingNode: (childNodes == null ? void 0 : childNodes.length) === index + 1,
      onDelete: onChildRemove,
      allowUserToDefineData: allowUserToDefineData,
      nodeLabel: nodeLabel
    }, textInputProps));
  }))));
};
var useStyles$c = /*#__PURE__*/core.makeStyles(function (theme) {
  return core.createStyles({
    edgeWrapper: {
      position: 'absolute',
      top: 0,
      bottom: 0,
      left: 0,
      width: theme.spacing(5),
      zIndex: 0
    },
    edgeVer: {
      width: 1,
      backgroundColor: theme.palette.grey['300']
    },
    edgeHor: {
      position: 'absolute',
      height: 1,
      width: '100%',
      backgroundColor: theme.palette.grey['300']
    },
    root: {
      position: 'relative',
      padding: theme.spacing(2, 0, 2, 5)
    },
    node: {
      position: 'relative',
      padding: theme.spacing(2),
      display: 'flex',
      flexDirection: 'column',
      width: 'fit-content'
    },
    payloadCaption: {
      marginRight: theme.spacing(1),
      width: theme.spacing(13)
    },
    payloadEntry: {
      display: 'flex',
      flexDirection: 'row',
      alignItems: 'center',
      borderBottomWidth: 1,
      borderBottomStyle: 'solid',
      borderBottomColor: theme.palette.grey['200']
    },
    removeButton: {
      position: 'absolute',
      height: 36,
      width: 36,
      top: -18,
      right: -18
    },
    addButton: {
      position: 'absolute',
      height: 36,
      width: 36,
      bottom: -18,
      right: -18
    }
  });
});

var TreeBuilder = function TreeBuilder(_ref) {
  var name = _ref.name,
      register = _ref.register,
      control = _ref.control,
      label = _ref.label,
      treeNode = _objectWithoutPropertiesLoose(_ref, ["name", "register", "control", "label", "error", "setValue", "unregister", "reset"]);

  return React__default.createElement("div", {
    style: {
      border: '1px transparent solid'
    }
  }, React__default.createElement(core.FormLabel, null, label), React__default.createElement(TreeNodeInput, Object.assign({}, treeNode, {
    id: uuid(),
    formContext: {
      name: name,
      register: register,
      control: control
    }
  })));
};

var FormArchitect = function FormArchitect(props) {
  var _useRfaDataConverter = useRfaDataConverter(),
      registerComponents = _useRfaDataConverter.handlers.registerComponents;

  var Theme = React__default.useMemo(function () {
    return createMuiTheme(_extends({}, props.theme));
  }, []);
  var formElements = React__default.useMemo(function () {
    var _props$formElements;

    return [{
      name: 'text-input',
      label: 'Text',
      description: 'for names, addresses, passwords, urls...',
      icon: React__default.createElement(icons.FormatColorTextOutlined, {
        color: 'secondary'
      }),
      render: TextInput,
      attributes: [{
        name: 'label',
        type: 'input',
        label: 'label',
        value: 'text label'
      }, {
        name: 'placeholder',
        type: 'input',
        label: 'placeholder',
        value: 'text placeholder'
      }, {
        name: 'helperText',
        type: 'input',
        label: 'helperText',
        value: 'helper text'
      }, {
        name: 'type',
        type: 'select',
        label: 'type',
        options: [{
          name: 'text'
        }, {
          name: 'password'
        }, {
          name: 'email'
        }],
        value: 'text'
      }],
      validationType: 'string',
      validators: ['required', 'min', 'max', 'length', 'matches', 'email', 'url']
    }, {
      name: 'number-input',
      label: 'Number',
      description: 'for ages, sums, counts...',
      icon: React__default.createElement(icons.Looks3Outlined, {
        color: 'secondary'
      }),
      render: NumberInput,
      attributes: [{
        name: 'label',
        type: 'input',
        label: 'label',
        value: 'number label'
      }, {
        name: 'placeholder',
        type: 'input',
        label: 'placeholder',
        value: 'number placeholder'
      }, {
        name: 'helperText',
        type: 'input',
        label: 'helperText',
        value: 'number helper text'
      }],
      validationType: 'number',
      validators: ['required', 'min', 'max', 'lessThan', 'moreThan', 'positive', 'negative', 'integer']
    }, {
      name: 'checkbox',
      label: 'Checkbox',
      description: 'for selecting one or more options',
      icon: React__default.createElement(icons.CheckBoxOutlined, {
        color: 'secondary'
      }),
      render: CheckBox,
      attributes: [{
        name: 'label',
        type: 'input',
        label: 'label',
        value: 'checkboxes label'
      }, {
        name: 'helperText',
        type: 'input',
        label: 'helperText',
        value: 'checkboxes helper text'
      }, {
        type: 'options',
        name: 'options',
        label: 'manage options',
        options: [{
          name: 'opt1'
        }, {
          name: 'opt2'
        }, {
          name: 'opt3'
        }]
      }],
      validationType: 'array',
      validators: ['required']
    }, {
      name: 'radio',
      label: 'Radio',
      description: 'for selecting one option',
      icon: React__default.createElement(icons.RadioButtonCheckedOutlined, {
        color: 'secondary'
      }),
      render: Radio,
      attributes: [{
        name: 'label',
        type: 'input',
        label: 'label',
        value: 'radio label'
      }, {
        name: 'helperText',
        type: 'input',
        label: 'helperText',
        value: 'radio helper text'
      }, {
        type: 'options',
        name: 'options',
        label: 'manage options',
        options: [{
          name: 'opt1'
        }, {
          name: 'opt2'
        }, {
          name: 'opt3'
        }]
      }],
      validationType: 'string',
      validators: ['required']
    }, {
      name: 'select',
      label: 'Select',
      description: 'for selecting one from numerous options',
      icon: React__default.createElement(icons.MenuOpenOutlined, {
        color: 'secondary'
      }),
      render: Select,
      attributes: [{
        name: 'label',
        type: 'input',
        label: 'label',
        value: 'select label'
      }, {
        name: 'helperText',
        type: 'input',
        label: 'helperText',
        value: 'select helper text'
      }, {
        type: 'options',
        name: 'options',
        label: 'manage options',
        options: [{
          name: 'opt1'
        }, {
          name: 'opt2'
        }, {
          name: 'opt3'
        }]
      }],
      validationType: 'string',
      validators: ['required']
    }, {
      name: 'multiSelect',
      label: 'Select - multiple options',
      description: 'for selecting one or more from numerous options',
      icon: React__default.createElement(icons.MenuOpenOutlined, {
        color: 'secondary'
      }),
      render: SelectMulti,
      attributes: [{
        name: 'label',
        type: 'input',
        label: 'label',
        value: 'multi select label'
      }, {
        name: 'helperText',
        type: 'input',
        label: 'helperText',
        value: 'multi select helper text'
      }, {
        type: 'options',
        name: 'options',
        label: 'manage options',
        options: [{
          name: 'opt1'
        }, {
          name: 'opt2'
        }, {
          name: 'opt3'
        }]
      }],
      validationType: 'array',
      validators: ['required']
    }, {
      name: 'switch',
      label: 'Switch',
      description: 'for simple yes or no answers',
      icon: React__default.createElement(icons.ToggleOnOutlined, {
        color: 'secondary'
      }),
      render: Switch,
      attributes: [{
        name: 'label',
        type: 'input',
        label: 'label',
        value: 'switch label'
      }, {
        name: 'helperText',
        type: 'input',
        label: 'helperText',
        value: 'switch helper text'
      }],
      validationType: 'boolean'
    }, {
      name: 'slider',
      label: 'Slider',
      description: 'for selecting a number interactively',
      icon: React__default.createElement(icons.LinearScaleOutlined, {
        color: 'secondary'
      }),
      render: Slider,
      isCustomRegistered: true,
      attributes: [{
        name: 'label',
        type: 'input',
        label: 'label',
        value: 'slider label'
      }, {
        name: 'min',
        type: 'input',
        label: 'Min Value',
        value: 0
      }, {
        name: 'max',
        type: 'input',
        label: 'Max Value',
        value: 100
      }, {
        name: 'step',
        type: 'input',
        label: 'Step',
        value: 1
      }, {
        name: 'helperText',
        type: 'input',
        label: 'helperText',
        value: 'slider helper text'
      }],
      validationType: 'number',
      validators: ['required', 'min', 'max', 'lessThan', 'moreThan', 'positive', 'negative', 'integer']
    }, {
      name: 'root',
      label: 'Tree',
      description: 'for structured data',
      icon: React__default.createElement(icons.AccountTreeOutlined, {
        color: 'secondary'
      }),
      render: TreeBuilder,
      validationType: 'object',
      attributes: [{
        name: 'label',
        type: 'input',
        label: 'tree title',
        value: 'Taxonomy of..'
      }, {
        name: 'nodeLabel',
        type: 'input',
        label: 'tree node label',
        value: 'node name'
      }, {
        name: 'placeholder',
        type: 'input',
        label: 'tree node label placeholder',
        value: 'eg. Cyanobacteria'
      }, {
        name: 'data',
        type: 'keyValueData',
        label: 'tree node payload',
        keyValueData: [{
          name: 'species',
          placeholder: 'vulpes vulpes'
        }, {
          name: 'genus',
          placeholder: 'vulpes'
        }, {
          name: 'phylum',
          placeholder: 'chordata'
        }, {
          name: 'class',
          placeholder: 'mammalia'
        }]
      }, {
        name: 'allowUserToDefineData',
        type: 'switch',
        label: 'Allow user to add more data',
        value: false
      }]
    }].concat((_props$formElements = props.formElements) != null ? _props$formElements : []);
  }, []); // added any additional components to store for consecutive form data export

  React__default.useEffect(function () {
    var _props$formElements2, _props$formElements3;

    if (!((_props$formElements2 = props.formElements) == null ? void 0 : _props$formElements2.length)) {
      return;
    }

    registerComponents((_props$formElements3 = props.formElements) == null ? void 0 : _props$formElements3.map(function (element) {
      return element.render;
    }));
  }, [props.formElements]);
  return React__default.createElement(reactDnd.DndProvider, {
    backend: reactDndHtml5Backend.HTML5Backend
  }, React__default.createElement(ThemeProvider, {
    theme: Theme
  }, React__default.createElement(CssBaseline, null), React__default.createElement(core.Grid, {
    container: true,
    direction: 'row',
    spacing: 1,
    justify: 'center'
  }, React__default.createElement(core.Grid, {
    item: true,
    xs: 9
  }, React__default.createElement(DropArea, null)), React__default.createElement(core.Grid, {
    item: true,
    xs: 3
  }, React__default.createElement(FieldBox, {
    formElements: formElements
  }))), React__default.createElement(core.Box, {
    marginTop: 1
  }, React__default.createElement(FormArchitectSaveButton, {
    onSave: props.onSave
  }))));
};

var FormComponentWrapper = function FormComponentWrapper(_ref) {
  var children = _ref.children,
      rest = _objectWithoutPropertiesLoose(_ref, ["children"]);

  return React__default.createElement(core.FormControl, Object.assign({
    style: {
      width: "100%"
    }
  }, rest), children);
};

var SortableWrapper = function SortableWrapper(_ref) {
  var id = _ref.id,
      index = _ref.index,
      swap = _ref.swap,
      children = _ref.children,
      dndIndicator = _ref.dndIndicator;
  var ref = React__default.useRef(null);

  var _useDrop = reactDnd.useDrop({
    accept: dndIndicator || "sortable",
    hover: function hover(item, monitor) {
      if (!ref.current) {
        return;
      }

      var dragIndex = item.index;
      var hoverIndex = index;

      if (dragIndex === hoverIndex) {
        return;
      }

      var hoverBoundingRect = ref.current.getBoundingClientRect();
      var hoverMiddleY = (hoverBoundingRect.bottom - hoverBoundingRect.top) / 2;
      var clientOffset = monitor.getClientOffset();
      var hoverClientY = clientOffset.y - hoverBoundingRect.top;

      if (dragIndex < hoverIndex && hoverClientY < hoverMiddleY) {
        return;
      }

      if (dragIndex > hoverIndex && hoverClientY > hoverMiddleY) {
        return;
      }

      swap(dragIndex, hoverIndex);
      item.index = hoverIndex;
    }
  }),
      drop = _useDrop[1];

  var _useDrag = reactDnd.useDrag({
    item: {
      type: dndIndicator || "sortable",
      id: id,
      index: index
    },
    collect: function collect(monitor) {
      return {
        isDragging: monitor.isDragging()
      };
    }
  }),
      isDragging = _useDrag[0].isDragging,
      drag = _useDrag[1];

  var opacity = isDragging ? 0.1 : 1;
  drag(drop(ref));
  return React__default.createElement("div", {
    ref: ref,
    style: {
      opacity: opacity
    }
  }, children);
};

var CheckBox = function CheckBox(props) {
  var _props$error;

  return React__default.createElement(FormComponentWrapper, {
    error: !!((_props$error = props.error) == null ? void 0 : _props$error.message)
  }, props.label && React__default.createElement(core.FormLabel, {
    component: "legend"
  }, props.label), React__default.createElement(core.FormGroup, null, props.options.map(function (option, i) {
    return React__default.createElement(core.FormControlLabel, {
      key: i,
      label: option.name,
      control: React__default.createElement(core.Checkbox, Object.assign({}, option.MuiProps, {
        inputRef: props.register,
        value: option.name,
        name: props.name
      }))
    });
  })), props.helperText && React__default.createElement(core.FormHelperText, null, props.helperText));
};

var Select = function Select(props) {
  var _props$error, _props$error2, _props$error3, _props$error$message, _props$error4;

  return React__default.createElement(FormComponentWrapper, {
    error: !!((_props$error = props.error) == null ? void 0 : _props$error.message)
  }, React__default.createElement(core.InputLabel, null, props.label), React__default.createElement(reactHookForm.Controller, {
    as: core.Select,
    multiple: props.multiple,
    name: props.name,
    control: props.control,
    defaultValue: props.multiple ? [] : ""
  }, props.options.map(function (option) {
    return React__default.createElement(core.MenuItem, {
      key: option.name,
      value: option.name
    }, option.name);
  })), (props.helperText || ((_props$error2 = props.error) == null ? void 0 : _props$error2.message)) && React__default.createElement(core.FormHelperText, {
    error: !!((_props$error3 = props.error) == null ? void 0 : _props$error3.message)
  }, (_props$error$message = (_props$error4 = props.error) == null ? void 0 : _props$error4.message) != null ? _props$error$message : props.helperText));
};

var SelectMulti = function SelectMulti(props) {
  return React__default.createElement(Select, Object.assign({}, props, {
    multiple: true
  }));
};

var NumberInput = function NumberInput(props) {
  return React__default.createElement(TextInput, Object.assign({}, props, {
    type: "number"
  }));
};

var Radio = function Radio(props) {
  var _props$error, _props$error2;

  return React__default.createElement(FormComponentWrapper, {
    error: !!((_props$error = props.error) == null ? void 0 : _props$error.message)
  }, React__default.createElement(core.FormLabel, {
    component: "legend"
  }, props.label), React__default.createElement(core.RadioGroup, {
    name: props.name,
    "aria-label": props.label
  }, props.options.map(function (option, i) {
    return React__default.createElement(core.FormControlLabel, {
      key: i,
      label: option.name,
      value: option.name,
      control: React__default.createElement(core.Radio, Object.assign({}, option.MuiProps, {
        inputRef: props.register
      }))
    });
  })), (props.helperText || ((_props$error2 = props.error) == null ? void 0 : _props$error2.message)) && React__default.createElement(core.FormHelperText, null, props.helperText));
};

var Slider = function Slider(_ref) {
  var _error$message;

  var label = _ref.label,
      name = _ref.name,
      helperText = _ref.helperText,
      unregister = _ref.unregister,
      register = _ref.register,
      setValue = _ref.setValue,
      error = _ref.error,
      sliderProps = _objectWithoutPropertiesLoose(_ref, ["label", "name", "helperText", "unregister", "register", "setValue", "error"]);

  //Custom form context registration with cleanup
  React__default.useEffect(function () {
    register({
      name: name
    });
    setValue(name, sliderProps.min);
    return function () {
      return unregister(name);
    };
  }, [name, register, setValue, sliderProps.min, unregister]);
  var handleChange = React__default.useCallback(function (_, value) {
    setValue(name, value);
  }, [name, setValue]);
  return React__default.createElement(FormComponentWrapper, {
    error: !!(error == null ? void 0 : error.message)
  }, React__default.createElement(core.FormLabel, {
    component: "legend",
    style: {
      marginBottom: theme.spacing(1)
    }
  }, label), React__default.createElement(core.Slider, Object.assign({}, sliderProps, {
    min: Number(sliderProps.min),
    max: Number(sliderProps.max),
    name: name,
    color: "secondary",
    "aria-labelledby": name,
    onChange: handleChange,
    valueLabelDisplay: "auto",
    defaultValue: Number(sliderProps.min),
    marks: [{
      label: sliderProps.min,
      value: Number(sliderProps.min)
    }, {
      label: sliderProps.max,
      value: Number(sliderProps.max)
    }]
  })), (helperText || (error == null ? void 0 : error.message)) && React__default.createElement(core.FormHelperText, {
    error: !!(error == null ? void 0 : error.message)
  }, (_error$message = error == null ? void 0 : error.message) != null ? _error$message : helperText));
};

var initialState$1 = {
  components: [TextInput, NumberInput, CheckBox, Radio, Select, SelectMulti, Switch, Slider, TreeBuilder]
};
var actions = {
  addComponents: function addComponents(store, components) {
    store.setState(_extends({}, store.state, {
      components: [].concat(store.state.components, components)
    }));
  }
};

var _createStore$1 = /*#__PURE__*/reactSimpleHookStore.createStore(initialState$1, actions),
    useComponentStore = _createStore$1.useStore;

var useRfaDataConverter = function useRfaDataConverter() {
  var _useComponentStore = useComponentStore(function (s) {
    return s.components;
  }, function (a) {
    return a.addComponents;
  }),
      components = _useComponentStore[0],
      addComponents = _useComponentStore[1];

  var dataToExport = React__default.useCallback(function (store) {
    return {
      grouping: store.grouping,
      elements: store.elements.map(function (element) {
        return _extends({}, element, {
          render: element.render.name
        });
      })
    };
  }, []);
  var dataToStore = React__default.useCallback(function (data) {
    return {
      grouping: data.grouping,
      elements: data.elements.map(function (element) {
        var _components$find;

        return _extends({}, element, {
          render: (_components$find = components.find(function (component) {
            return component.name === element.render;
          })) != null ? _components$find : function () {
            return null;
          }
        });
      })
    };
  }, [components]);
  var registerComponents = React__default.useCallback(function (components) {
    addComponents(components);
  }, [components]);
  var isExportedJson = React__default.useCallback(function (data) {
    return typeof data.elements[0].render === 'string';
  }, []);
  return {
    handlers: {
      dataToExport: dataToExport,
      dataToStore: dataToStore,
      registerComponents: registerComponents
    },
    state: {
      isExportedJson: isExportedJson
    }
  };
};

var useFormRenderer = function useFormRenderer(data) {
  var Elements = React__default.useMemo(function () {
    return data.elements;
  }, [data]);
  var Groups = React__default.useMemo(function () {
    return data.grouping.groups;
  }, [data]);
  var getGroupName = React__default.useCallback(function (groupId) {
    var _Groups$find;

    return (_Groups$find = Groups.find(function (group) {
      return group.id === groupId;
    })) == null ? void 0 : _Groups$find.name;
  }, [Groups]);
  var GroupReducedElements = React__default.useMemo(function () {
    return Elements.reduce(function (acc, el) {
      var _getGroupName, _acc$key, _extends2;

      var key = (_getGroupName = getGroupName(el.groupID)) != null ? _getGroupName : '';
      return _extends({}, acc, (_extends2 = {}, _extends2[key] = [].concat((_acc$key = acc[key]) != null ? _acc$key : [], [el]), _extends2));
    }, {});
  }, [Elements, getGroupName]);
  var shouldRenderGroups = React__default.useMemo(function () {
    return Groups.length > 1;
  }, [Groups]);
  return {
    data: {
      GroupReducedElements: GroupReducedElements
    },
    state: {
      shouldRenderGroups: shouldRenderGroups
    }
  };
};

var FormRendererForm = function FormRendererForm(_ref) {
  var data = _ref.data,
      onSubmit = _ref.onSubmit;

  var _useValidationCreator = useValidationCreator(),
      ValidationSchema = _useValidationCreator.data.ValidationSchema;

  var _useForm = reactHookForm.useForm({
    resolver: yup$1.yupResolver(ValidationSchema)
  }),
      handleSubmit = _useForm.handleSubmit,
      register = _useForm.register,
      unregister = _useForm.unregister,
      control = _useForm.control,
      setValue = _useForm.setValue,
      errors = _useForm.errors;

  var _useFormRenderer = useFormRenderer(data),
      GroupReducedElements = _useFormRenderer.data.GroupReducedElements,
      shouldRenderGroups = _useFormRenderer.state.shouldRenderGroups;

  var classes = useStyles$d();
  return React__default.createElement("form", {
    onSubmit: handleSubmit(onSubmit)
  }, React__default.createElement(core.Box, null, Object.entries(GroupReducedElements).map(function (_ref2, index) {
    var groupName = _ref2[0],
        elements = _ref2[1];
    return React__default.createElement(core.Box, {
      id: 'form-group',
      key: index,
      className: classes.group
    }, shouldRenderGroups && React__default.createElement(React__default.Fragment, null, React__default.createElement(core.Typography, {
      variant: 'h5',
      gutterBottom: true
    }, groupName)), React__default.createElement(core.Paper, {
      variant: 'outlined',
      id: 'form-group-content',
      className: classes.groupContent
    }, elements.map(function (element) {
      return React__default.createElement(core.Box, {
        id: 'form-element',
        key: element.id,
        className: classes.formElement
      }, React__default.createElement(element.render, _extends({}, {
        register: register,
        unregister: unregister,
        control: control,
        setValue: setValue
      }, element.attributes, {
        name: element.name,
        error: errors == null ? void 0 : errors[element.name]
      })));
    })));
  })), React__default.createElement(core.Button, {
    variant: 'contained',
    type: 'submit'
  }, "submit"));
};
var useStyles$d = /*#__PURE__*/core.makeStyles(function (theme) {
  return core.createStyles({
    group: {
      paddingBottom: theme.spacing(3)
    },
    groupContent: {
      padding: theme.spacing(2)
    },
    formElement: {
      marginTop: theme.spacing(1.5),
      '&:nth-child(1)': {
        marginTop: 0
      }
    }
  });
});

var FormRenderer = function FormRenderer(_ref) {
  var data = _ref.data,
      onSubmit = _ref.onSubmit,
      theme = _ref.theme,
      fields = _ref.fields;

  var _useRfaDataConverter = useRfaDataConverter(),
      registerComponents = _useRfaDataConverter.handlers.registerComponents; // register any additional form fields


  React__default.useEffect(function () {
    if (!fields) {
      return;
    }

    registerComponents(fields);
  }, [fields]);
  var Theme = React__default.useMemo(function () {
    return createMuiTheme(_extends({}, theme));
  }, []);

  var _useRfaDataConverter2 = useRfaDataConverter(),
      dataToStore = _useRfaDataConverter2.handlers.dataToStore,
      isExportedJson = _useRfaDataConverter2.state.isExportedJson; //convert data to correct type before render


  var FormRendererData = React__default.useMemo(function () {
    return isExportedJson(data) ? dataToStore(data) : data;
  }, [data, dataToStore]);
  return React__default.createElement(core.ThemeProvider, {
    theme: Theme
  }, React__default.createElement(core.CssBaseline, null), React__default.createElement(core.Container, {
    maxWidth: 'xl'
  }, FormRendererData && React__default.createElement(FormRendererForm, {
    data: FormRendererData,
    onSubmit: onSubmit
  })));
};

var TreeNodeTitle = function TreeNodeTitle(_ref) {
  var label = _ref.label,
      rest = _objectWithoutPropertiesLoose(_ref, ["label"]);

  return React__default.createElement(core.Typography, Object.assign({
    color: "primary",
    variant: "h6"
  }, rest), label);
};

var TreeNode = function TreeNode(_ref) {
  var _nodeRect$height, _nodeRect$height2, _node$name, _node$userData, _node$descendents2;

  var node = _ref.node,
      _ref$isEndingNode = _ref.isEndingNode,
      isEndingNode = _ref$isEndingNode === void 0 ? false : _ref$isEndingNode,
      _ref$level = _ref.level,
      level = _ref$level === void 0 ? 1 : _ref$level,
      _ref$isCollapsed = _ref.isCollapsed,
      isCollapsed = _ref$isCollapsed === void 0 ? false : _ref$isCollapsed,
      onSelect = _ref.onSelect;
  var classes = useStyles$e();

  var _React$useState = React__default.useState(!isCollapsed),
      isOpen = _React$useState[0],
      setIsOpen = _React$useState[1];

  var _useIntersectionObser = useIntersectionObserver(),
      ref = _useIntersectionObser.ref,
      isSticky = _useIntersectionObser.isSticky;

  var _useClientRect = useClientRect(),
      nodeRect = _useClientRect[0],
      nodeRef = _useClientRect[1],
      nodeElement = _useClientRect[2];

  var hasDescendents = React__default.useMemo(function () {
    var _node$descendents;

    return !!((_node$descendents = node.descendents) == null ? void 0 : _node$descendents.length);
  }, [node.descendents]);
  var toggleDescendents = React__default.useCallback(function () {
    if (!hasDescendents) {
      return;
    }

    setIsOpen(function (prevState) {
      return !prevState;
    });
  }, [hasDescendents]);
  var scrollToNode = React__default.useCallback(function () {
    if (!nodeElement) {
      return;
    }

    nodeElement.scrollIntoView({
      behavior: 'smooth'
    });
  }, [nodeElement]);
  var handleSelect = React__default.useCallback(function () {
    onSelect == null ? void 0 : onSelect(node);
  }, [node, onSelect]);
  return React__default.createElement(React__default.Fragment, null, React__default.createElement("div", {
    className: classes.root,
    style: level === 1 ? {
      paddingLeft: 0
    } : {}
  }, level !== 1 && React__default.createElement("div", {
    className: classes.edgeWrapper
  }, React__default.createElement("div", {
    className: classes.edgeVer,
    style: {
      height: !isEndingNode ? '100%' : ((_nodeRect$height = nodeRect == null ? void 0 : nodeRect.height) != null ? _nodeRect$height : 2) / 2 + theme.spacing(2)
    }
  }), React__default.createElement("div", {
    className: classes.edgeHor,
    style: {
      top: ((_nodeRect$height2 = nodeRect == null ? void 0 : nodeRect.height) != null ? _nodeRect$height2 : 2) / 2 + theme.spacing(2)
    }
  })), React__default.createElement(NodeTitleAnchor, {
    visible: !!(isSticky && hasDescendents && isOpen),
    title: (_node$name = node.name) != null ? _node$name : "Node level " + level,
    onClick: scrollToNode
  }), React__default.createElement(core.Paper, {
    variant: 'outlined',
    className: classes.node,
    innerRef: nodeRef
  }, React__default.createElement(core.Box, {
    display: 'flex',
    flexDirection: 'column',
    alignItems: 'flex-start',
    justifyContent: 'center'
  }, React__default.createElement(TreeNodeTitle, {
    label: node.name,
    gutterBottom: !!node.data
  }), hasDescendents && React__default.createElement(TreeNodeCornerButton, {
    onClick: toggleDescendents,
    icon: isOpen ? 'expandLess' : 'expandMore',
    corner: 'bottomLeft',
    color: 'action'
  }), React__default.createElement("div", {
    ref: ref
  }), node.data && Object.entries(node.data).map(function (_ref2, index) {
    var name = _ref2[0],
        value = _ref2[1];
    return React__default.createElement(core.Box, {
      className: classes.payloadEntry,
      key: index
    }, React__default.createElement(core.Typography, {
      variant: 'caption',
      className: classes.payloadCaption
    }, name + ":"), React__default.createElement(core.Typography, {
      variant: 'body2'
    }, value));
  }), (_node$userData = node.userData) == null ? void 0 : _node$userData.map(function (dataEntry, index) {
    return React__default.createElement(core.Box, {
      className: classes.payloadEntry,
      key: index
    }, React__default.createElement(core.Typography, {
      variant: 'caption',
      className: classes.payloadCaption
    }, dataEntry.name + ":"), React__default.createElement(core.Typography, {
      variant: 'body2'
    }, dataEntry.value));
  })), onSelect && React__default.createElement(core.Box, {
    display: 'flex',
    alignItems: 'center',
    justifyContent: 'center',
    pl: 4
  }, React__default.createElement(core.IconButton, {
    color: "default",
    "aria-label": "select data",
    onClick: handleSelect
  }, React__default.createElement(icons.ArrowForward, null)))), React__default.createElement(core.Collapse, {
    "in": isOpen
  }, (_node$descendents2 = node.descendents) == null ? void 0 : _node$descendents2.map(function (descendant, index) {
    var _node$descendents3;

    return React__default.createElement(TreeNode, {
      key: index,
      node: descendant,
      level: level + 1,
      isEndingNode: ((_node$descendents3 = node.descendents) == null ? void 0 : _node$descendents3.length) === index + 1,
      onSelect: onSelect,
      isCollapsed: isCollapsed
    });
  }))));
};
var useStyles$e = /*#__PURE__*/styles.makeStyles(function (theme) {
  return core.createStyles({
    edgeWrapper: {
      position: 'absolute',
      top: 0,
      bottom: 0,
      left: 0,
      width: theme.spacing(5),
      zIndex: 0
    },
    edgeVer: {
      width: 1,
      backgroundColor: theme.palette.grey['300']
    },
    edgeHor: {
      position: 'absolute',
      height: 1,
      width: '100%',
      backgroundColor: theme.palette.grey['300']
    },
    root: {
      position: 'relative',
      padding: theme.spacing(2, 0, 2, 5)
    },
    node: {
      position: 'relative',
      padding: theme.spacing(2),
      display: 'flex',
      flexDirection: 'row',
      width: 'fit-content'
    },
    payloadCaption: {
      marginRight: theme.spacing(1),
      width: theme.spacing(13)
    },
    payloadEntry: {
      display: 'flex',
      flexDirection: 'row',
      alignItems: 'center',
      borderBottomWidth: 1,
      borderBottomStyle: 'solid',
      borderBottomColor: theme.palette.grey['200']
    }
  });
});

var SearchBar = function SearchBar(props) {
  var classes = useStyles$f();
  var inputRef = React__default.useRef(null);

  var _React$useState = React__default.useState(null),
      anchorEl = _React$useState[0],
      setAnchorEl = _React$useState[1];

  var handleChange = function handleChange(event) {
    var _extends2;

    props.onSearchInChange(_extends({}, props.searchIn, (_extends2 = {}, _extends2[event.target.name] = event.target.checked, _extends2)));
  };

  var handleMatchingChange = function handleMatchingChange(event) {
    props.onMatchPartial(event.target.checked);
  };

  var handleOpenMenu = React__default.useCallback(function (event) {
    setAnchorEl(anchorEl ? null : event.currentTarget);
  }, [anchorEl]);
  var open = React__default.useMemo(function () {
    return Boolean(anchorEl);
  }, [anchorEl]);
  var id = open ? 'simple-popper' : undefined;
  var handleSubmit = React__default.useCallback(function (event) {
    var _inputRef$current;

    event.preventDefault();
    props.onSearch((_inputRef$current = inputRef.current) == null ? void 0 : _inputRef$current.value);
  }, [props]);
  console.log(open);
  return React__default.createElement(Paper, {
    component: "form",
    className: classes.root,
    onSubmit: handleSubmit
  }, React__default.createElement(IconButton, {
    className: classes.iconButton,
    "aria-label": "menu",
    onClick: handleOpenMenu
  }, React__default.createElement(MenuIcon, null)), React__default.createElement(Popper, {
    id: id,
    open: open,
    anchorEl: anchorEl,
    placement: 'bottom-start'
  }, React__default.createElement(Paper, null, React__default.createElement(core.Box, {
    p: 1
  }, React__default.createElement(core.FormControl, {
    component: "fieldset"
  }, React__default.createElement(core.FormGroup, null, React__default.createElement(core.FormControlLabel, {
    control: React__default.createElement(core.Checkbox, {
      checked: props.searchIn.label,
      onChange: handleChange,
      name: "label"
    }),
    label: "Search in labels"
  }), React__default.createElement(core.FormControlLabel, {
    control: React__default.createElement(core.Checkbox, {
      checked: props.searchIn.content,
      onChange: handleChange,
      name: "content"
    }),
    label: "Search in content"
  }))), React__default.createElement(core.Divider, null), React__default.createElement(core.FormControlLabel, {
    control: React__default.createElement(core.Checkbox, {
      checked: props.matchPartial,
      onChange: handleMatchingChange,
      name: "partial"
    }),
    label: "Match partial results"
  })))), React__default.createElement(InputBase, {
    className: classes.input,
    name: 'search',
    placeholder: "Search Tree",
    inputProps: {
      'aria-label': 'search tree node'
    },
    inputRef: inputRef
  }), React__default.createElement(IconButton, {
    type: "submit",
    className: classes.iconButton,
    "aria-label": "search"
  }, React__default.createElement(SearchIcon, null)), React__default.createElement(core.Divider, {
    className: classes.divider,
    orientation: "vertical"
  }), React__default.createElement(IconButton, {
    onClick: function onClick() {
      return props.onSearch(undefined);
    },
    className: classes.iconButton,
    "aria-label": "clear"
  }, React__default.createElement(icons.BackspaceOutlined, null)));
};
var useStyles$f = /*#__PURE__*/styles$1.makeStyles(function (theme) {
  return styles$1.createStyles({
    root: {
      padding: '2px 4px',
      display: 'flex',
      alignItems: 'center',
      width: 400
    },
    input: {
      marginLeft: theme.spacing(1),
      flex: 1
    },
    iconButton: {
      padding: 10
    },
    divider: {
      height: 28,
      margin: 4
    }
  });
});

var Tree = function Tree(_ref) {
  var theme = _ref.theme,
      node = _ref.node,
      searchable = _ref.searchable,
      onNodeSelect = _ref.onNodeSelect;
  var Theme = React__default.useMemo(function () {
    return createMuiTheme(_extends({}, theme));
  }, []);

  var _React$useState = React__default.useState(''),
      query = _React$useState[0],
      setQuery = _React$useState[1];

  var _React$useState2 = React__default.useState([Node]),
      tree = _React$useState2[0],
      setTree = _React$useState2[1];

  var _React$useState3 = React__default.useState({
    content: false,
    label: true
  }),
      searchIn = _React$useState3[0],
      setSearchIn = _React$useState3[1];

  var _React$useState4 = React__default.useState(false),
      matchPartial = _React$useState4[0],
      setMatchPartial = _React$useState4[1];

  var _React$useState5 = React__default.useState(false),
      isLoading = _React$useState5[0],
      setLoading = _React$useState5[1];

  React__default.useEffect(function () {
    if (!query) {
      setQuery('');
      return setTree([node]);
    }

    setLoading(true);
    var result = searchForMany(node, query, [].concat(searchIn.content ? ['content'] : [], searchIn.label ? ['label'] : []), matchPartial);
    setTree(result);
    setLoading(false);
  }, [matchPartial, query, searchIn.content, searchIn.label]);
  return React__default.createElement(core.ThemeProvider, {
    theme: Theme
  }, React__default.createElement(core.CssBaseline, null), React__default.createElement(core.Container, null, React__default.createElement("div", {
    style: {
      border: '1px transparent solid'
    }
  }, searchable && React__default.createElement(SearchBar, {
    onSearch: setQuery,
    searchIn: searchIn,
    onSearchInChange: setSearchIn,
    matchPartial: matchPartial,
    onMatchPartial: setMatchPartial
  }), isLoading && React__default.createElement(core.LinearProgress, null), tree && tree.length ? tree.map(function (node, i) {
    return React__default.createElement("div", {
      key: i
    }, React__default.createElement(TreeNode, {
      node: node,
      isCollapsed: true,
      onSelect: onNodeSelect
    }), React__default.createElement(core.Divider, null));
  }) : React__default.createElement(core.Typography, null, "No results match query."))));
};

var searchForMany = function searchForMany(rootNode, query, searchIn, partialResults) {
  console.log('searching for ', query, 'in fields', searchIn);
  var queue = [rootNode];
  var results = [];

  while (queue.length) {
    var _node$descendents;

    var node = queue.shift();

    if (node == null ? void 0 : (_node$descendents = node.descendents) == null ? void 0 : _node$descendents.length) {
      queue.push.apply(queue, node.descendents);
    }

    if (searchIn.includes('label') && ((node == null ? void 0 : node.name) === query || partialResults && (node == null ? void 0 : node.name.includes(query)))) {
      results.push(node);
    } else {
      if (searchIn.includes('content')) {
        if (!(node == null ? void 0 : node.data)) {
          continue;
        }

        var res = Object.values(node.data).find(function (value) {
          return value.toString().toLowerCase() === query.toLowerCase() || partialResults && value.includes(query);
        });

        if (res) {
          results.push(node);
        }
      }
    }
  }

  return results.length ? results : null;
};

exports.FormArchitect = FormArchitect;
exports.FormRenderer = FormRenderer;
exports.RfaSchemaTransferer = RfaSchemaTransferer;
exports.Tree = Tree;
//# sourceMappingURL=rfa.cjs.development.js.map
